var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Geometry/models/billboard.ts
var billboard_exports = {};
__export(billboard_exports, {
  default: () => BillBoardModel
});
module.exports = __toCommonJS(billboard_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_billboard_frag = __toESM(require("../shaders/billboard_frag.glsl"));
var import_billboard_vert = __toESM(require("../shaders/billboard_vert.glsl"));
var BillBoardModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.radian = 0;
    this.planeGeometryTriangulation = () => {
      const {
        center = [120, 30]
      } = this.layer.getLayerConfig();
      return {
        size: 4,
        indices: [0, 1, 2, 2, 3, 0],
        vertices: [
          ...center,
          ...[1, 1],
          ...center,
          ...[0, 1],
          ...center,
          ...[0, 0],
          ...center,
          ...[1, 0]
        ]
      };
    };
  }
  getUninforms() {
    const {
      opacity,
      width = 1,
      height = 1,
      raisingHeight = 0
    } = this.layer.getLayerConfig();
    let rotateFlag = 1;
    if (this.mapService.version === "GAODE2.x" || this.mapService.version === "GAODE1.x") {
      rotateFlag = -1;
    }
    this.radian = rotateFlag * Math.PI * (this.mapService.getRotation() % 360) / 180;
    return {
      u_raisingHeight: Number(raisingHeight),
      u_RotateMatrix: new Float32Array([
        Math.cos(this.radian),
        Math.sin(this.radian),
        -Math.sin(this.radian),
        Math.cos(this.radian)
      ]),
      u_opacity: opacity || 1,
      u_texture: this.texture,
      u_size: [width, height]
    };
  }
  clearModels() {
    var _a;
    (_a = this.texture) == null ? void 0 : _a.destroy();
  }
  initModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      drawCanvas
    } = this.layer.getLayerConfig();
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    if (drawCanvas) {
      this.updateTexture(drawCanvas);
    }
    this.layer.buildLayerModel({
      moduleName: "geometryBillboard",
      vertexShader: import_billboard_vert.default,
      fragmentShader: import_billboard_frag.default,
      triangulation: this.planeGeometryTriangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: true },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  buildModels(callbackModel) {
    this.initModels(callbackModel);
  }
  updateTexture(drawCanvas) {
    const { createTexture2D } = this.rendererService;
    const {
      canvasWidth = 1,
      canvasHeight = 1
    } = this.layer.getLayerConfig();
    const canvas = document.createElement("canvas");
    canvas.width = canvasWidth;
    canvas.height = canvasHeight;
    const ctx = canvas.getContext("2d");
    if (ctx) {
      drawCanvas(canvas);
      this.texture = createTexture2D({
        data: canvas,
        width: canvas.width,
        height: canvas.height,
        wrapS: import_l7_core.gl.CLAMP_TO_EDGE,
        wrapT: import_l7_core.gl.CLAMP_TO_EDGE
      });
      this.layerService.updateLayerRenderList();
      this.layerService.renderLayers();
    }
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "extrude",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Extrude",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const extrude = [1, 1, 0, -1, 1, 0, -1, -1, 0, 1, -1, 0];
          const extrudeIndex = attributeIdx % 4 * 3;
          return [
            extrude[extrudeIndex],
            extrude[extrudeIndex + 1],
            extrude[extrudeIndex + 2]
          ];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[2], vertex[3]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
