var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Geometry/models/plane.ts
var plane_exports = {};
__export(plane_exports, {
  default: () => PlaneModel
});
module.exports = __toCommonJS(plane_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_maps = require("@antv/l7-maps");
var import_l7_utils = require("@antv/l7-utils");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_plane_frag = __toESM(require("../shaders/plane_frag.glsl"));
var import_plane_vert = __toESM(require("../shaders/plane_vert.glsl"));
var PlaneModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.terrainImageLoaded = false;
    this.planeGeometryTriangulation = () => {
      const {
        width = 1,
        height = 1,
        widthSegments = 1,
        heightSegments = 1,
        center = [120, 30],
        terrainTexture
      } = this.layer.getLayerConfig();
      const { indices, positions } = this.initPlane(width, height, widthSegments, heightSegments, ...center);
      if (terrainTexture) {
        this.loadTerrainTexture(positions, indices);
      }
      return {
        vertices: positions,
        indices,
        size: 5
      };
    };
  }
  initPlane(width = 1, height = 1, widthSegments = 1, heightSegments = 1, lng = 120, lat = 30) {
    const widthHalf = width / 2;
    const heightHalf = height / 2;
    const gridX = Math.floor(widthSegments);
    const gridY = Math.floor(heightSegments);
    const gridX1 = gridX + 1;
    const gridY1 = gridY + 1;
    const segmentWidth = width / gridX;
    const segmentHeight = height / gridY;
    const indices = [];
    const positions = [];
    for (let iy = 0; iy < gridY1; iy++) {
      const y = iy * segmentHeight - heightHalf;
      for (let ix = 0; ix < gridX1; ix++) {
        const x = ix * segmentWidth - widthHalf;
        if (this.mapService.version === import_l7_maps.Version["GAODE2.x"]) {
          const [a, b] = this.mapService.lngLatToCoord([x + lng, -y + lat]);
          positions.push(a, b, 0);
        } else {
          positions.push(x + lng, -y + lat, 0);
        }
        positions.push(ix / gridX);
        positions.push(1 - iy / gridY);
      }
    }
    for (let iy = 0; iy < gridY; iy++) {
      for (let ix = 0; ix < gridX; ix++) {
        const a = ix + gridX1 * iy;
        const b = ix + gridX1 * (iy + 1);
        const c = ix + 1 + gridX1 * (iy + 1);
        const d = ix + 1 + gridX1 * iy;
        indices.push(a, b, d);
        indices.push(b, c, d);
      }
    }
    return { indices, positions };
  }
  getUninforms() {
    var _a;
    const {
      opacity,
      mapTexture,
      terrainClipHeight = 0,
      terrainTexture
    } = this.layer.getLayerConfig();
    if (this.mapTexture !== mapTexture) {
      this.mapTexture = mapTexture;
      (_a = this.texture) == null ? void 0 : _a.destroy();
      this.updateTexture(mapTexture);
    }
    return {
      u_opacity: opacity || 1,
      u_mapFlag: mapTexture ? 1 : 0,
      u_terrainClipHeight: terrainTexture ? terrainClipHeight : -1,
      u_texture: this.texture
    };
  }
  clearModels() {
    var _a;
    this.terrainImage = null;
    (_a = this.texture) == null ? void 0 : _a.destroy();
  }
  initModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      mapTexture
    } = this.layer.getLayerConfig();
    this.mapTexture = mapTexture;
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    this.updateTexture(mapTexture);
    this.layer.buildLayerModel({
      moduleName: "geometryPlane",
      vertexShader: import_plane_vert.default,
      fragmentShader: import_plane_frag.default,
      triangulation: this.planeGeometryTriangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: true },
      stencil: (0, import_l7_utils.getMask)(mask, maskInside),
      blend: this.getBlend(),
      cull: {
        enable: true,
        face: import_l7_core.gl.BACK
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  buildModels(callbackModel) {
    this.initModels(callbackModel);
  }
  createModelData(options) {
    if (options) {
      const {
        widthSegments: oldwidthSegments,
        heightSegments: oldheightSegments,
        width: oldwidth,
        height: oldheight
      } = this.layer.getLayerConfig();
      const {
        widthSegments,
        heightSegments,
        width,
        height
      } = options;
      this.layer.style({
        widthSegments: widthSegments !== void 0 ? widthSegments : oldwidthSegments,
        heightSegments: heightSegments !== void 0 ? heightSegments : oldheightSegments,
        width: width !== void 0 ? width : oldwidth,
        height: height !== void 0 ? height : oldheight
      });
    }
    const oldFeatures = this.layer.getEncodedData();
    const res = this.styleAttributeService.createAttributesAndIndices(oldFeatures, this.planeGeometryTriangulation);
    return res;
  }
  updateTexture(mapTexture) {
    const { createTexture2D } = this.rendererService;
    if (mapTexture) {
      const img = new Image();
      img.crossOrigin = "anonymous";
      img.onload = () => {
        this.texture = createTexture2D({
          data: img,
          width: img.width,
          height: img.height,
          wrapS: import_l7_core.gl.CLAMP_TO_EDGE,
          wrapT: import_l7_core.gl.CLAMP_TO_EDGE
        });
        this.layerService.updateLayerRenderList();
        this.layerService.renderLayers();
      };
      img.src = mapTexture;
    } else {
      this.texture = createTexture2D({
        width: 0,
        height: 0
      });
    }
  }
  getImageData(img) {
    const canvas = document.createElement("canvas");
    const ctx = canvas.getContext("2d");
    const { width, height } = img;
    canvas.width = width;
    canvas.height = height;
    ctx.drawImage(img, 0, 0, width, height);
    const imageData = ctx.getImageData(0, 0, width, height);
    return imageData;
  }
  translateVertex(positions, indices, image, widthSegments, heightSegments, rgb2height) {
    const imgWidth = image.width;
    const imgHeight = image.height;
    const imageData = this.getImageData(image).data;
    const gridX = Math.floor(widthSegments);
    const gridY = Math.floor(heightSegments);
    const gridX1 = gridX + 1;
    const gridY1 = gridY + 1;
    const widthStep = imgWidth / gridX;
    const heihgtStep = imgHeight / gridY;
    for (let iy = 0; iy < gridY1; iy++) {
      const imgIndexY = Math.floor(iy * heihgtStep);
      const imgLen = imgIndexY * imgWidth;
      for (let ix = 0; ix < gridX1; ix++) {
        const imgIndexX = Math.floor(ix * widthStep);
        const imgDataIndex = (imgLen + imgIndexX) * 4;
        const r = imageData[imgDataIndex];
        const g = imageData[imgDataIndex + 1];
        const b = imageData[imgDataIndex + 2];
        const z = (iy * gridX1 + ix) * 5 + 2;
        positions[z] = rgb2height(r, g, b);
      }
    }
    const oldFeatures = this.layer.getEncodedData();
    const modelData = this.styleAttributeService.createAttributesAndIndices(oldFeatures, () => {
      return {
        vertices: positions,
        indices,
        size: 5
      };
    });
    this.layer.updateModelData(modelData);
    this.layerService.renderLayers();
  }
  loadTerrainTexture(positions, indices) {
    const {
      widthSegments = 1,
      heightSegments = 1,
      terrainTexture,
      rgb2height = (r, g, b) => r + g + b
    } = this.layer.getLayerConfig();
    if (this.terrainImage) {
      if (this.terrainImageLoaded) {
        this.translateVertex(positions, indices, this.terrainImage, widthSegments, heightSegments, rgb2height);
      } else {
        this.terrainImage.onload = () => {
          this.translateVertex(positions, indices, this.terrainImage, widthSegments, heightSegments, rgb2height);
        };
      }
    } else {
      const terrainImage = new Image();
      this.terrainImage = terrainImage;
      terrainImage.crossOrigin = "anonymous";
      terrainImage.onload = () => {
        this.terrainImageLoaded = true;
        setTimeout(() => this.layer.emit("terrainImageLoaded", null));
        this.translateVertex(positions, indices, terrainImage, widthSegments, heightSegments, rgb2height);
      };
      terrainImage.src = terrainTexture;
    }
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
