var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Geometry/models/sprite.ts
var sprite_exports = {};
__export(sprite_exports, {
  default: () => SpriteModel
});
module.exports = __toCommonJS(sprite_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_maps = require("@antv/l7-maps");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_sprite_frag = __toESM(require("../shaders/sprite_frag.glsl"));
var import_sprite_vert = __toESM(require("../shaders/sprite_vert.glsl"));
var SpriteModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.planeGeometryUpdateTriangulation = () => {
      const {
        spriteBottom = -1e5
      } = this.layer.getLayerConfig();
      const updateZ = this.spriteUpdate;
      const bottomZ = spriteBottom;
      const topZ = this.spriteTop;
      for (let i = 0; i < this.positions.length; i += 5) {
        if (this.spriteAnimate === "up" /* UP */) {
          this.positions[i + 2] += updateZ;
          if (this.positions[i + 2] > topZ) {
            this.positions[i + 2] = bottomZ;
          }
        } else {
          this.positions[i + 2] -= updateZ;
          if (this.positions[i + 2] < bottomZ) {
            this.positions[i + 2] = topZ;
          }
        }
      }
      return {
        vertices: this.positions,
        indices: this.indices,
        size: 5
      };
    };
    this.updateModel = () => {
      const attributes = this.layer.createAttrubutes({
        triangulation: this.planeGeometryUpdateTriangulation
      });
      this.layer.models.map((m) => {
        m.updateAttributes(attributes);
      });
      this.layer.renderLayers();
      this.timer = requestAnimationFrame(this.updateModel);
    };
    this.planeGeometryTriangulation = () => {
      const {
        center = [120, 30],
        spriteCount = 100,
        spriteRadius = 10
      } = this.layer.getLayerConfig();
      const { indices, positions } = this.initSprite(spriteRadius, spriteCount, ...center);
      this.positions = positions;
      this.indices = indices;
      return {
        vertices: positions,
        indices,
        size: 5
      };
    };
  }
  initSprite(radius = 10, spriteCount = 100, lng = 120, lat = 30) {
    const indices = [];
    const positions = [];
    const mapService = this.mapService;
    const heightLimit = this.spriteAnimate === "up" /* UP */ ? -this.spriteTop : this.spriteTop;
    for (let i = 0; i < spriteCount; i++) {
      const height = Math.random() * heightLimit;
      positions.push(...getPos(height));
    }
    for (let i = 0; i < spriteCount; i++) {
      indices.push(i);
    }
    function getPos(z) {
      const randomX = radius * Math.random();
      const randomY = radius * Math.random();
      const x = -radius / 2 + randomX;
      const y = -radius / 2 + randomY;
      if (mapService.version === import_l7_maps.Version["GAODE2.x"]) {
        const [a, b] = mapService.lngLatToCoord([x + lng, -y + lat]);
        return [a, b, z, 0, 0];
      } else {
        return [x + lng, -y + lat, z, 0, 0];
      }
    }
    return { indices, positions };
  }
  getUninforms() {
    var _a;
    const {
      opacity,
      mapTexture,
      spriteScale = 1
    } = this.layer.getLayerConfig();
    if (this.mapTexture !== mapTexture) {
      this.mapTexture = mapTexture;
      (_a = this.texture) == null ? void 0 : _a.destroy();
      this.updateTexture(mapTexture);
    }
    return {
      u_opacity: opacity || 1,
      u_mapFlag: mapTexture ? 1 : 0,
      u_texture: this.texture,
      u_Scale: spriteScale
    };
  }
  clearModels() {
    var _a;
    cancelAnimationFrame(this.timer);
    (_a = this.texture) == null ? void 0 : _a.destroy();
  }
  initModels(callbackModel) {
    const {
      mapTexture,
      spriteTop = 5e6,
      spriteUpdate = 1e4,
      spriteAnimate = "down" /* DOWN */
    } = this.layer.getLayerConfig();
    this.mapTexture = mapTexture;
    this.spriteTop = spriteTop;
    this.spriteUpdate = spriteUpdate;
    spriteAnimate === "up" ? this.spriteAnimate = "up" /* UP */ : this.spriteAnimate = "down" /* DOWN */;
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    this.updateTexture(mapTexture);
    setTimeout(() => {
      this.updateModel();
    }, 100);
    this.layer.buildLayerModel({
      moduleName: "geometrySprite",
      vertexShader: import_sprite_vert.default,
      fragmentShader: import_sprite_frag.default,
      triangulation: this.planeGeometryTriangulation,
      primitive: import_l7_core.gl.POINTS,
      depth: { enable: false },
      blend: this.getBlend()
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  buildModels(callbackModel) {
    this.initModels(callbackModel);
  }
  updateTexture(mapTexture) {
    const { createTexture2D } = this.rendererService;
    if (mapTexture) {
      const img = new Image();
      img.crossOrigin = "anonymous";
      img.onload = () => {
        this.texture = createTexture2D({
          data: img,
          width: img.width,
          height: img.height,
          wrapS: import_l7_core.gl.CLAMP_TO_EDGE,
          wrapT: import_l7_core.gl.CLAMP_TO_EDGE
        });
        this.layerService.updateLayerRenderList();
        this.layerService.renderLayers();
      };
      img.src = mapTexture;
    } else {
      this.texture = createTexture2D({
        width: 0,
        height: 0
      });
    }
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  registerBuiltinAttributes() {
    return "";
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
