var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/canvas/models/canvas.ts
var canvas_exports = {};
__export(canvas_exports, {
  default: () => CanvaModel
});
module.exports = __toCommonJS(canvas_exports);
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_interface = require("../../core/interface");
var CanvaModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.updateMode = import_interface.CanvasUpdateType.ALWAYS;
    this.renderUpdate = () => {
      const {
        zIndex = 10,
        update = import_interface.CanvasUpdateType.ALWAYS,
        animateOption = { enable: false, duration: 20 }
      } = this.layer.getLayerConfig();
      if (+this.canvas.style.zIndex === zIndex) {
        this.canvas.style.zIndex = zIndex + "";
      }
      if (this.updateMode !== update) {
        this.updateMode = update;
        this.unBindListener();
        this.bindListener();
      }
      if (this.updateMode === import_interface.CanvasUpdateType.ALWAYS && animateOption.enable) {
        this.renderCanvas();
      }
    };
    this.unBindListener = () => {
      this.mapService.off("mapchange", this.renderCanvas);
      this.mapService.off("zoomstart", this.clearCanvas);
      this.mapService.off("zoomend", this.renderCanvas);
      this.mapService.off("movestart", this.clearCanvas);
      this.mapService.off("moveend", this.renderCanvas);
    };
    this.bindListener = () => {
      if (this.updateMode === import_interface.CanvasUpdateType.ALWAYS) {
        this.mapService.on("mapchange", this.renderCanvas);
      } else {
        this.mapService.on("zoomstart", this.clearCanvas);
        this.mapService.on("zoomend", this.renderCanvas);
        this.mapService.on("movestart", this.clearCanvas);
        this.mapService.on("moveend", this.renderCanvas);
      }
    };
    this.clearCanvas = () => {
      if (this.ctx) {
        const { width: w, height: h } = this.rendererService.getViewportSize();
        this.ctx.clearRect(0, 0, w, h);
      }
    };
    this.renderCanvas = () => {
      const {
        width: viewWidth,
        height: viewHeight
      } = this.rendererService.getViewportSize();
      if (this.prevSize[0] !== viewWidth || this.prevSize[1] !== viewHeight) {
        this.prevSize = [viewWidth, viewHeight];
        const size = this.mapService.getSize();
        const [width, height] = size;
        this.canvas.width = viewWidth;
        this.canvas.height = viewHeight;
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
      }
      const {
        drawingOnCanvas
      } = this.layer.getLayerConfig();
      if (this.ctx) {
        drawingOnCanvas({
          canvas: this.canvas,
          ctx: this.ctx,
          mapService: this.mapService,
          size: [viewWidth, viewHeight]
        });
      }
    };
  }
  clearModels() {
    var _a;
    if (this.canvas) {
      (_a = this.mapService.getContainer()) == null ? void 0 : _a.removeChild(this.canvas);
      this.canvas = null;
    }
    this.unBindListener();
  }
  initModels() {
    const {
      update = import_interface.CanvasUpdateType.ALWAYS
    } = this.layer.getLayerConfig();
    this.updateMode = update;
    this.initCanvas();
    this.renderCanvas();
    this.bindListener();
    this.mapService.getContainer();
    return [];
  }
  initCanvas() {
    var _a;
    const { zIndex } = this.layer.getLayerConfig();
    const size = this.mapService.getSize();
    const [width, height] = size;
    const {
      width: viewWidth,
      height: viewHeight
    } = this.rendererService.getViewportSize();
    this.prevSize = [viewWidth, viewHeight];
    const canvas = document.createElement("canvas");
    this.canvas = canvas;
    canvas.width = viewWidth;
    canvas.height = viewHeight;
    canvas.style.pointerEvents = "none";
    canvas.style.width = width + "px";
    canvas.style.height = height + "px";
    canvas.style.position = "absolute";
    canvas.style.top = "0";
    canvas.style.left = "0";
    canvas.style.zIndex = zIndex + "";
    (_a = this.mapService.getContainer()) == null ? void 0 : _a.appendChild(canvas);
    const ctx = canvas.getContext("2d");
    this.ctx = ctx;
  }
  buildModels() {
    return this.initModels();
  }
  registerBuiltinAttributes() {
    return;
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
