var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/citybuliding/models/build.ts
var build_exports = {};
__export(build_exports, {
  default: () => CityBuildModel
});
module.exports = __toCommonJS(build_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_build_frag = __toESM(require("../shaders/build_frag.glsl"));
var import_build_vert = __toESM(require("../shaders/build_vert.glsl"));
var CityBuildModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      baseColor = "rgb(16,16,16)",
      brightColor = "rgb(255,176,38)",
      windowColor = "rgb(30,60,89)",
      time = 0,
      sweep = {
        enable: false,
        sweepRadius: 1,
        sweepColor: "rgb(255, 255, 255)",
        sweepSpeed: 0.4,
        sweepCenter: this.cityCenter
      }
    } = this.layer.getLayerConfig();
    return {
      u_cityCenter: sweep.sweepCenter || this.cityCenter,
      u_cityMinSize: this.cityMinSize * sweep.sweepRadius,
      u_circleSweep: sweep.enable ? 1 : 0,
      u_circleSweepColor: (0, import_l7_utils.rgb2arr)(sweep.sweepColor).slice(0, 3),
      u_circleSweepSpeed: sweep.sweepSpeed,
      u_opacity: opacity,
      u_baseColor: (0, import_l7_utils.rgb2arr)(baseColor),
      u_brightColor: (0, import_l7_utils.rgb2arr)(brightColor),
      u_windowColor: (0, import_l7_utils.rgb2arr)(windowColor),
      u_time: this.layer.getLayerAnimateTime() || time
    };
  }
  calCityGeo() {
    const [minLng, minLat, maxLng, maxLat] = this.layer.getSource().extent;
    if (this.mapService.version === "GAODE2.x") {
      this.cityCenter = this.mapService.lngLatToCoord([
        (maxLng + minLng) / 2,
        (maxLat + minLat) / 2
      ]);
      const l1 = this.mapService.lngLatToCoord([maxLng, maxLat]);
      const l2 = this.mapService.lngLatToCoord([minLng, minLat]);
      this.cityMinSize = Math.sqrt(Math.pow(l1[0] - l2[0], 2) + Math.pow(l1[1] - l2[1], 2)) / 4;
    } else {
      const w = maxLng - minLng;
      const h = maxLat - minLat;
      this.cityCenter = [(maxLng + minLng) / 2, (maxLat + minLat) / 2];
      this.cityMinSize = Math.sqrt(Math.pow(w, 2) + Math.pow(h, 2)) / 4;
    }
  }
  initModels(callbackModel) {
    this.calCityGeo();
    this.startModelAnimate();
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    this.layer.buildLayerModel({
      moduleName: "cityBuilding",
      vertexShader: import_build_vert.default,
      fragmentShader: import_build_frag.default,
      triangulation: import_triangulation.PolygonExtrudeTriangulation,
      depth: { enable: true },
      cull: {
        enable: true,
        face: import_l7_core.gl.BACK
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 10 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size } = feature;
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
