var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};

// src/core/BaseModel.ts
var BaseModel_exports = {};
__export(BaseModel_exports, {
  default: () => BaseModel
});
module.exports = __toCommonJS(BaseModel_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_d3_color = require("d3-color");
var import_lodash = require("lodash");
var import_blend = require("../utils/blend");
var BaseModel = class {
  constructor(layer) {
    this.layer = layer;
    this.rendererService = layer.getContainer().get(import_l7_core.TYPES.IRendererService);
    this.pickingService = layer.getContainer().get(import_l7_core.TYPES.IPickingService);
    this.shaderModuleService = layer.getContainer().get(import_l7_core.TYPES.IShaderModuleService);
    this.styleAttributeService = layer.getContainer().get(import_l7_core.TYPES.IStyleAttributeService);
    this.mapService = layer.getContainer().get(import_l7_core.TYPES.IMapService);
    this.iconService = layer.getContainer().get(import_l7_core.TYPES.IIconService);
    this.fontService = layer.getContainer().get(import_l7_core.TYPES.IFontService);
    this.cameraService = layer.getContainer().get(import_l7_core.TYPES.ICameraService);
    this.layerService = layer.getContainer().get(import_l7_core.TYPES.ILayerService);
    this.registerBuiltinAttributes();
    this.startModelAnimate();
    const { createTexture2D } = this.rendererService;
    this.createTexture2D = createTexture2D;
    this.DATA_TEXTURE_WIDTH = 1024;
    this.rowCount = 1;
    this.cellLength = 0;
    this.cellProperties = [];
    this.cacheStyleProperties = {
      thetaOffset: void 0,
      opacity: void 0,
      strokeOpacity: void 0,
      strokeWidth: void 0,
      stroke: void 0,
      offsets: void 0
    };
    this.stylePropertyesExist = {
      hasThetaOffset: 0,
      hasOpacity: 0,
      hasStrokeOpacity: 0,
      hasStrokeWidth: 0,
      hasStroke: 0,
      hasOffsets: 0
    };
    this.dataTextureTest = this.layerService.getOESTextureFloat();
    if (!this.dataTextureTest) {
      this.dataTexture = this.createTexture2D({
        data: new Uint8Array(4),
        mag: import_l7_core.gl.NEAREST,
        min: import_l7_core.gl.NEAREST,
        width: 1,
        height: 1
      });
    }
  }
  clearLastCalRes() {
    this.cellProperties = [];
    this.cellLength = 0;
    this.stylePropertyesExist = {
      hasThetaOffset: 0,
      hasOpacity: 0,
      hasStrokeOpacity: 0,
      hasStrokeWidth: 0,
      hasStroke: 0,
      hasOffsets: 0
    };
  }
  getCellTypeLayout() {
    if (this.dataTextureTest) {
      return [
        this.rowCount,
        this.DATA_TEXTURE_WIDTH,
        0,
        0,
        this.stylePropertyesExist.hasOpacity,
        this.stylePropertyesExist.hasStrokeOpacity,
        this.stylePropertyesExist.hasStrokeWidth,
        this.stylePropertyesExist.hasStroke,
        this.stylePropertyesExist.hasOffsets,
        this.stylePropertyesExist.hasThetaOffset,
        0,
        0,
        0,
        0,
        0,
        1
      ];
    } else {
      return [
        1,
        1,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        -1
      ];
    }
  }
  dataTextureNeedUpdate(options) {
    let isUpdate = false;
    if (!(0, import_lodash.isEqual)(options.thetaOffset, this.cacheStyleProperties.thetaOffset)) {
      isUpdate = true;
      this.cacheStyleProperties.thetaOffset = options.thetaOffset;
    }
    if (!(0, import_lodash.isEqual)(options.opacity, this.cacheStyleProperties.opacity)) {
      isUpdate = true;
      this.cacheStyleProperties.opacity = options.opacity;
    }
    if (!(0, import_lodash.isEqual)(options.strokeOpacity, this.cacheStyleProperties.strokeOpacity)) {
      isUpdate = true;
      this.cacheStyleProperties.strokeOpacity = options.strokeOpacity;
    }
    if (!(0, import_lodash.isEqual)(options.strokeWidth, this.cacheStyleProperties.strokeWidth)) {
      isUpdate = true;
      this.cacheStyleProperties.strokeWidth = options.strokeWidth;
    }
    if (!(0, import_lodash.isEqual)(options.stroke, this.cacheStyleProperties.stroke)) {
      isUpdate = true;
      this.cacheStyleProperties.stroke = options.stroke;
    }
    if (!(0, import_lodash.isEqual)(options.offsets, this.cacheStyleProperties.offsets)) {
      isUpdate = true;
      this.cacheStyleProperties.offsets = options.offsets;
    }
    if (this.dataTexture === void 0) {
      isUpdate = true;
    }
    return isUpdate;
  }
  judgeStyleAttributes(options) {
    this.clearLastCalRes();
    if (options.opacity !== void 0 && !(0, import_lodash.isNumber)(options.opacity)) {
      this.cellProperties.push({ attr: "opacity", count: 1 });
      this.stylePropertyesExist.hasOpacity = 1;
      this.cellLength += 1;
    }
    if (options.strokeOpacity !== void 0 && !(0, import_lodash.isNumber)(options.strokeOpacity)) {
      this.cellProperties.push({ attr: "strokeOpacity", count: 1 });
      this.stylePropertyesExist.hasStrokeOpacity = 1;
      this.cellLength += 1;
    }
    if (options.strokeWidth !== void 0 && !(0, import_lodash.isNumber)(options.strokeWidth)) {
      this.cellProperties.push({ attr: "strokeWidth", count: 1 });
      this.stylePropertyesExist.hasStrokeWidth = 1;
      this.cellLength += 1;
    }
    if (options.stroke !== void 0 && !this.isStaticColor(options.stroke)) {
      this.cellProperties.push({ attr: "stroke", count: 4 });
      this.stylePropertyesExist.hasStroke = 1;
      this.cellLength += 4;
    }
    if (options.offsets !== void 0 && !this.isOffsetStatic(options.offsets)) {
      this.cellProperties.push({ attr: "offsets", count: 2 });
      this.stylePropertyesExist.hasOffsets = 1;
      this.cellLength += 2;
    }
    if (options.thetaOffset !== void 0 && !(0, import_lodash.isNumber)(options.thetaOffset)) {
      this.cellProperties.push({ attr: "thetaOffset", count: 1 });
      this.stylePropertyesExist.hasThetaOffset = 1;
      this.cellLength += 1;
    }
  }
  isStaticColor(stroke) {
    if ((0, import_lodash.isString)(stroke)) {
      if ((0, import_d3_color.color)(stroke)) {
        return true;
      } else {
        return false;
      }
    }
    return false;
  }
  getStrokeColor(stroke) {
    if (this.isStaticColor(stroke)) {
      const strokeColor = (0, import_l7_utils.rgb2arr)(stroke);
      strokeColor[0] = strokeColor[0] ? strokeColor[0] : 0;
      strokeColor[1] = strokeColor[1] ? strokeColor[1] : 0;
      strokeColor[2] = strokeColor[2] ? strokeColor[2] : 0;
      strokeColor[3] = strokeColor[3] ? strokeColor[3] : 0;
      return strokeColor;
    } else {
      return [0, 0, 0, 0];
    }
  }
  isOffsetStatic(offsets) {
    if (Array.isArray(offsets) && offsets.length === 2 && (0, import_lodash.isNumber)(offsets[0]) && (0, import_lodash.isNumber)(offsets[1])) {
      return true;
    } else {
      return false;
    }
  }
  patchMod(d, count) {
    for (let i = 0; i < count; i++) {
      d.push(-1);
    }
  }
  patchData(d, cellData, cellPropertiesLayouts) {
    for (const layout of cellPropertiesLayouts) {
      const { attr, count } = layout;
      const value = cellData[attr];
      if (value !== void 0) {
        if (attr === "stroke") {
          d.push(...(0, import_l7_utils.rgb2arr)(value));
        } else if (attr === "offsets") {
          if (this.isOffsetStatic(value)) {
            d.push(-value[0], value[1]);
          } else {
            d.push(0, 0);
          }
        } else {
          d.push((0, import_lodash.isNumber)(value) ? value : 1);
        }
      } else {
        this.patchMod(d, count);
      }
    }
  }
  calDataFrame(cellLength, encodeData, cellPropertiesLayouts) {
    const encodeDatalength = encodeData.length;
    const rowCount = Math.ceil(encodeDatalength * cellLength / this.DATA_TEXTURE_WIDTH);
    const totalLength = rowCount * this.DATA_TEXTURE_WIDTH;
    const d = [];
    for (let i = 0; i < encodeDatalength; i++) {
      const cellData = encodeData[i];
      this.patchData(d, cellData, cellPropertiesLayouts);
    }
    for (let i = d.length; i < totalLength; i++) {
      d.push(-1);
    }
    return { data: d, width: this.DATA_TEXTURE_WIDTH, height: rowCount };
  }
  getBlend() {
    const { blend = "normal" } = this.layer.getLayerConfig();
    return import_blend.BlendTypes[import_l7_core.BlendType[blend]];
  }
  getDefaultStyle() {
    return {};
  }
  getUninforms() {
    throw new Error("Method not implemented.");
  }
  getAnimateUniforms() {
    return {};
  }
  needUpdate() {
    return false;
  }
  buildModels(callbackModel) {
    throw new Error("Method not implemented.");
  }
  initModels(callbackModel) {
    throw new Error("Method not implemented.");
  }
  clearModels() {
    return;
  }
  getAttribute() {
    throw new Error("Method not implemented.");
  }
  render() {
    throw new Error("Method not implemented.");
  }
  registerBuiltinAttributes() {
    throw new Error("Method not implemented.");
  }
  animateOption2Array(option) {
    return [
      option.enable ? 0 : 1,
      option.duration || 4,
      option.interval || 0.2,
      option.trailLength || 0.1
    ];
  }
  startModelAnimate() {
    const { animateOption } = this.layer.getLayerConfig();
    if (animateOption.enable) {
      this.layer.setAnimateStartTime();
    }
  }
};
__decorateClass([
  (0, import_l7_core.lazyInject)(import_l7_core.TYPES.IGlobalConfigService)
], BaseModel.prototype, "configService", 2);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
