var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/core/shape/extrude.ts
var extrude_exports = {};
__export(extrude_exports, {
  default: () => extrudePolygon,
  extrude_PolygonNormal: () => extrude_PolygonNormal,
  fillPolygon: () => fillPolygon
});
module.exports = __toCommonJS(extrude_exports);
var import_l7_utils = require("@antv/l7-utils");
var import_earcut = __toESM(require("earcut"));
var import_gl_matrix = require("gl-matrix");
function extrudePolygon(path) {
  const p1 = path[0][0];
  const p2 = path[0][path[0].length - 1];
  if (p1[0] === p2[0] && p1[1] === p2[1]) {
    path[0] = path[0].slice(0, path[0].length - 1);
  }
  const n = path[0].length;
  const flattengeo = import_earcut.default.flatten(path);
  const { vertices, dimensions } = flattengeo;
  const positions = [];
  const indexArray = [];
  for (let j = 0; j < vertices.length / dimensions; j++) {
    if (dimensions === 2) {
      positions.push(vertices[j * 2], vertices[j * 2 + 1], 1);
    } else {
      positions.push(vertices[j * 3], vertices[j * 3 + 1], 1);
    }
  }
  const triangles = (0, import_earcut.default)(flattengeo.vertices, flattengeo.holes, flattengeo.dimensions);
  indexArray.push(...triangles);
  for (let i = 0; i < n; i++) {
    const prePoint = flattengeo.vertices.slice(i * dimensions, (i + 1) * dimensions);
    let nextPoint = flattengeo.vertices.slice((i + 1) * dimensions, (i + 2) * dimensions);
    if (nextPoint.length === 0) {
      nextPoint = flattengeo.vertices.slice(0, dimensions);
    }
    const indexOffset = positions.length / 3;
    positions.push(prePoint[0], prePoint[1], 1, nextPoint[0], nextPoint[1], 1, prePoint[0], prePoint[1], 0, nextPoint[0], nextPoint[1], 0);
    indexArray.push(...[0, 2, 1, 2, 3, 1].map((v) => v + indexOffset));
  }
  return {
    positions,
    index: indexArray
  };
}
function fillPolygon(points) {
  const flattengeo = import_earcut.default.flatten(points);
  const triangles = (0, import_earcut.default)(flattengeo.vertices, flattengeo.holes, flattengeo.dimensions);
  return {
    positions: flattengeo.vertices,
    index: triangles
  };
}
function extrude_PolygonNormal(path, needFlat = false) {
  const p1 = path[0][0];
  const p2 = path[0][path[0].length - 1];
  if (p1[0] === p2[0] && p1[1] === p2[1]) {
    path[0] = path[0].slice(0, path[0].length - 1);
  }
  const n = path[0].length;
  const flattengeo = import_earcut.default.flatten(path);
  const { vertices, dimensions } = flattengeo;
  const positions = [];
  const indexArray = [];
  const normals = [];
  for (let j = 0; j < vertices.length / dimensions; j++) {
    if (dimensions === 2) {
      positions.push(vertices[j * 2], vertices[j * 2 + 1], 1, -1, -1);
    } else {
      positions.push(vertices[j * 3], vertices[j * 3 + 1], 1, -1, -1);
    }
    normals.push(0, 0, 1);
  }
  const triangles = (0, import_earcut.default)(flattengeo.vertices, flattengeo.holes, flattengeo.dimensions);
  indexArray.push(...triangles);
  for (let i = 0; i < n; i++) {
    const prePoint = flattengeo.vertices.slice(i * dimensions, (i + 1) * dimensions);
    let nextPoint = flattengeo.vertices.slice((i + 1) * dimensions, (i + 2) * dimensions);
    if (nextPoint.length === 0) {
      nextPoint = flattengeo.vertices.slice(0, dimensions);
    }
    const indexOffset = positions.length / 5;
    positions.push(prePoint[0], prePoint[1], 1, 0, 0, nextPoint[0], nextPoint[1], 1, 0.1, 0, prePoint[0], prePoint[1], 0, 0, 0.8, nextPoint[0], nextPoint[1], 0, 0.1, 0.8);
    const normal = computeVertexNormals([nextPoint[0], nextPoint[1], 1], [prePoint[0], prePoint[1], 0], [prePoint[0], prePoint[1], 1], needFlat);
    normals.push(...normal, ...normal, ...normal, ...normal);
    indexArray.push(...[1, 2, 0, 3, 2, 1].map((v) => v + indexOffset));
  }
  return {
    positions,
    index: indexArray,
    normals
  };
}
function computeVertexNormals(p1, p2, p3, needFlat = false) {
  const cb = import_gl_matrix.vec3.create();
  const ab = import_gl_matrix.vec3.create();
  const normal = import_gl_matrix.vec3.create();
  if (needFlat) {
    p1 = (0, import_l7_utils.lngLatToMeters)(p1);
    p2 = (0, import_l7_utils.lngLatToMeters)(p2);
    p3 = (0, import_l7_utils.lngLatToMeters)(p3);
  }
  const pA = import_gl_matrix.vec3.fromValues(...p1);
  const pB = import_gl_matrix.vec3.fromValues(...p2);
  const pC = import_gl_matrix.vec3.fromValues(...p3);
  import_gl_matrix.vec3.sub(cb, pC, pB);
  import_gl_matrix.vec3.sub(ab, pA, pB);
  import_gl_matrix.vec3.cross(normal, cb, ab);
  const newNormal = import_gl_matrix.vec3.create();
  import_gl_matrix.vec3.normalize(newNormal, normal);
  return newNormal;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  extrude_PolygonNormal,
  fillPolygon
});
