var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/earth/models/base.ts
var base_exports = {};
__export(base_exports, {
  default: () => BaseEarthModel
});
module.exports = __toCommonJS(base_exports);
var import_l7_core = require("@antv/l7-core");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_base_frag = __toESM(require("../shaders/base_frag.glsl"));
var import_base_vert = __toESM(require("../shaders/base_vert.glsl"));
var BaseEarthModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.earthTime = 3.4;
    this.sunX = 1e3;
    this.sunY = 1e3;
    this.sunZ = 1e3;
    this.sunRadius = Math.sqrt(this.sunX * this.sunX + this.sunY * this.sunY + this.sunZ * this.sunZ);
  }
  getUninforms() {
    const { animateOption, globelOtions } = this.layer.getLayerConfig();
    if (animateOption == null ? void 0 : animateOption.enable) {
      this.mapService.rotateY({
        reg: 2e-3
      });
      this.earthTime += 0.02;
      this.sunY = 10;
      this.sunX = Math.cos(this.earthTime) * (this.sunRadius - this.sunY);
      this.sunZ = Math.sin(this.earthTime) * (this.sunRadius - this.sunY);
    }
    return {
      u_ambientRatio: (globelOtions == null ? void 0 : globelOtions.ambientRatio) || 0.6,
      u_diffuseRatio: (globelOtions == null ? void 0 : globelOtions.diffuseRatio) || 0.4,
      u_specularRatio: (globelOtions == null ? void 0 : globelOtions.specularRatio) || 0.1,
      u_sunLight: [this.sunX, this.sunY, this.sunZ],
      u_texture: this.texture
    };
  }
  setEarthTime(time) {
    this.earthTime = time;
    this.sunY = 10;
    this.sunX = Math.cos(this.earthTime) * (this.sunRadius - this.sunY);
    this.sunZ = Math.sin(this.earthTime) * (this.sunRadius - this.sunY);
    this.layerService.renderLayers();
  }
  initModels(callbackModel) {
    const { globelOtions } = this.layer.getLayerConfig();
    if ((globelOtions == null ? void 0 : globelOtions.earthTime) !== void 0) {
      this.setEarthTime(globelOtions.earthTime);
    }
    const source = this.layer.getSource();
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    source.data.images.then((imageData) => {
      this.texture = createTexture2D({
        data: imageData[0],
        width: imageData[0].width,
        height: imageData[0].height
      });
      this.layerService.updateLayerRenderList();
      this.layerService.renderLayers();
    });
    this.buildModels(callbackModel);
  }
  clearModels() {
    return "";
  }
  buildModels(callbackModel) {
    this.layer.zIndex = -998;
    this.layer.buildLayerModel({
      moduleName: "earthBase",
      vertexShader: import_base_vert.default,
      fragmentShader: import_base_frag.default,
      triangulation: import_triangulation.earthTriangulation,
      depth: { enable: true },
      blend: this.getBlend()
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 1 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
