var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/earth/utils.ts
var utils_exports = {};
__export(utils_exports, {
  EARTH_RADIUS: () => EARTH_RADIUS,
  EARTH_RADIUS_OUTER: () => EARTH_RADIUS_OUTER,
  EARTH_SEGMENTS: () => EARTH_SEGMENTS,
  lglt2xyz: () => lglt2xyz,
  primitiveSphere: () => primitiveSphere
});
module.exports = __toCommonJS(utils_exports);
var import_gl_matrix = require("gl-matrix");
var EARTH_RADIUS = 100;
var EARTH_SEGMENTS = 36;
var EARTH_RADIUS_OUTER = 40;
function torad(deg) {
  return deg / 180 * Math.acos(-1);
}
function lglt2xyz(lnglat) {
  const lng = torad(lnglat[0]) + Math.PI / 2;
  const lat = torad(lnglat[1]);
  const radius = EARTH_RADIUS + Math.random() * 0.4;
  const z = radius * Math.cos(lat) * Math.cos(lng);
  const x = radius * Math.cos(lat) * Math.sin(lng);
  const y = radius * Math.sin(lat);
  return [x, y, z];
}
function primitiveSphere(radius, opt) {
  const matRotY = import_gl_matrix.mat4.create();
  const matRotZ = import_gl_matrix.mat4.create();
  const up = import_gl_matrix.vec3.fromValues(0, 1, 0);
  const tmpVec3 = import_gl_matrix.vec3.fromValues(0, 0, 0);
  opt = opt || {};
  radius = typeof radius !== "undefined" ? radius : 1;
  const segments = typeof opt.segments !== "undefined" ? opt.segments : 32;
  const totalZRotationSteps = 2 + segments;
  const totalYRotationSteps = 2 * totalZRotationSteps;
  const indices = [];
  const indicesArr = [];
  const positions = [];
  const positionsArr = [];
  const normalArr = [];
  const uvs = [];
  for (let zRotationStep = 0; zRotationStep <= totalZRotationSteps; zRotationStep++) {
    const normalizedZ = zRotationStep / totalZRotationSteps;
    const angleZ = normalizedZ * Math.PI;
    for (let yRotationStep = 0; yRotationStep <= totalYRotationSteps; yRotationStep++) {
      const normalizedY = yRotationStep / totalYRotationSteps;
      const angleY = normalizedY * Math.PI * 2;
      import_gl_matrix.mat4.identity(matRotZ);
      import_gl_matrix.mat4.rotateZ(matRotZ, matRotZ, -angleZ);
      import_gl_matrix.mat4.identity(matRotY);
      import_gl_matrix.mat4.rotateY(matRotY, matRotY, angleY);
      import_gl_matrix.vec3.transformMat4(tmpVec3, up, matRotZ);
      import_gl_matrix.vec3.transformMat4(tmpVec3, tmpVec3, matRotY);
      import_gl_matrix.vec3.scale(tmpVec3, tmpVec3, -radius);
      positions.push(tmpVec3.slice());
      positionsArr.push(...tmpVec3.slice());
      import_gl_matrix.vec3.normalize(tmpVec3, tmpVec3);
      normalArr.push(...tmpVec3.slice());
      uvs.push([normalizedY, 1 - normalizedZ]);
      positionsArr.push(normalizedY, 1 - normalizedZ);
    }
    if (zRotationStep > 0) {
      const verticesCount = positions.length;
      let firstIndex = verticesCount - 2 * (totalYRotationSteps + 1);
      for (; firstIndex + totalYRotationSteps + 2 < verticesCount; firstIndex++) {
        indices.push([
          firstIndex,
          firstIndex + 1,
          firstIndex + totalYRotationSteps + 1
        ]);
        indicesArr.push(firstIndex, firstIndex + 1, firstIndex + totalYRotationSteps + 1);
        indices.push([
          firstIndex + totalYRotationSteps + 1,
          firstIndex + 1,
          firstIndex + totalYRotationSteps + 2
        ]);
        indicesArr.push(firstIndex + totalYRotationSteps + 1, firstIndex + 1, firstIndex + totalYRotationSteps + 2);
      }
    }
  }
  return {
    cells: indices,
    positions,
    uvs,
    positionsArr,
    indicesArr,
    normalArr
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  EARTH_RADIUS,
  EARTH_RADIUS_OUTER,
  EARTH_SEGMENTS,
  lglt2xyz,
  primitiveSphere
});
