var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/line/models/half.ts
var half_exports = {};
__export(half_exports, {
  default: () => LineModel
});
module.exports = __toCommonJS(half_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_line_half_frag = __toESM(require("../shaders/half/line_half_frag.glsl"));
var import_line_half_vert = __toESM(require("../shaders/half/line_half_vert.glsl"));
var LineModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      sourceColor,
      targetColor,
      arrow = {
        enable: false,
        arrowWidth: 2,
        arrowHeight: 3,
        tailWidth: 1
      }
    } = this.layer.getLayerConfig();
    let useLinearColor = 0;
    let sourceColorArr = [0, 0, 0, 0];
    let targetColorArr = [0, 0, 0, 0];
    if (sourceColor && targetColor) {
      sourceColorArr = (0, import_l7_utils.rgb2arr)(sourceColor);
      targetColorArr = (0, import_l7_utils.rgb2arr)(targetColor);
      useLinearColor = 1;
    }
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_linearColor: useLinearColor,
      u_sourceColor: sourceColorArr,
      u_targetColor: targetColorArr,
      u_arrow: Number(arrow.enable),
      u_arrowHeight: arrow.arrowHeight || 3,
      u_arrowWidth: arrow.arrowWidth || 2,
      u_tailWidth: arrow.tailWidth === void 0 ? 1 : arrow.tailWidth
    };
  }
  initModels(callbackModel) {
    this.buildModels(callbackModel);
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  async buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      depth = false
    } = this.layer.getLayerConfig();
    const { frag, vert } = this.getShaders();
    this.layer.triangulation = import_triangulation.LineTriangulation;
    this.layer.buildLayerModel({
      moduleName: "lineHalf",
      vertexShader: vert,
      fragmentShader: frag,
      triangulation: import_triangulation.LineTriangulation,
      depth: { enable: depth },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  getShaders() {
    return {
      frag: import_line_half_frag.default,
      vert: import_line_half_vert.default,
      type: "normal"
    };
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "distanceAndIndex",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_DistanceAndIndex",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal, vertexIndex) => {
          return vertexIndex === void 0 ? [vertex[3], 10, vertex[5]] : [vertex[3], vertexIndex, vertex[5]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "dirPoints",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_dirPoints",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 4,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const startPoint = feature.coordinates[0] || [0, 0];
          const endPoint = feature.coordinates[3] || [0, 0];
          return [startPoint[0], startPoint[1], endPoint[0], endPoint[1]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 1 } = feature;
          return Array.isArray(size) ? [size[0], size[1]] : [size, 0];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "miter",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Miter",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[4]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
