var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/line/models/line.ts
var line_exports = {};
__export(line_exports, {
  default: () => LineModel
});
module.exports = __toCommonJS(line_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_interface = require("../../core/interface");
var import_line_dash_frag = __toESM(require("../shaders/dash/line_dash_frag.glsl"));
var import_line_dash_vert = __toESM(require("../shaders/dash/line_dash_vert.glsl"));
var import_line_frag = __toESM(require("../shaders/line_frag.glsl"));
var import_line_vert = __toESM(require("../shaders/line_vert.glsl"));
var import_line_linear_frag = __toESM(require("../shaders/linear/line_linear_frag.glsl"));
var lineStyleObj = {
  solid: 0,
  dash: 1
};
var LineModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.updateTexture = () => {
      const { createTexture2D } = this.rendererService;
      if (this.texture) {
        this.texture.update({
          data: this.iconService.getCanvas()
        });
        this.layer.render();
        return;
      }
      this.texture = createTexture2D({
        data: this.iconService.getCanvas(),
        mag: import_l7_core.gl.NEAREST,
        min: import_l7_core.gl.NEAREST,
        premultiplyAlpha: false,
        width: 1024,
        height: this.iconService.canvasHeight || 128
      });
    };
  }
  getUninforms() {
    const {
      opacity = 1,
      sourceColor,
      targetColor,
      textureBlend = "normal",
      lineType = "solid",
      dashArray = [10, 5, 0, 0],
      lineTexture = false,
      iconStep = 100,
      vertexHeightScale = 20,
      borderWidth = 0,
      borderColor = "#ccc",
      raisingHeight = 0,
      heightfixed = false,
      linearDir = import_interface.LinearDir.VERTICAL,
      blur = [1, 1, 1],
      arrow = {
        enable: false,
        arrowWidth: 2,
        arrowHeight: 3,
        tailWidth: 1
      }
    } = this.layer.getLayerConfig();
    if (dashArray.length === 2) {
      dashArray.push(0, 0);
    }
    if (this.rendererService.getDirty()) {
      this.texture.bind();
    }
    let useLinearColor = 0;
    let sourceColorArr = [0, 0, 0, 0];
    let targetColorArr = [0, 0, 0, 0];
    if (sourceColor && targetColor) {
      sourceColorArr = (0, import_l7_utils.rgb2arr)(sourceColor);
      targetColorArr = (0, import_l7_utils.rgb2arr)(targetColor);
      useLinearColor = 1;
    }
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_textureBlend: textureBlend === import_interface.TextureBlend.NORMAL ? 0 : 1,
      u_line_type: lineStyleObj[lineType],
      u_dash_array: dashArray,
      u_blur: blur,
      u_texture: this.texture,
      u_line_texture: lineTexture ? 1 : 0,
      u_icon_step: iconStep,
      u_textSize: [1024, this.iconService.canvasHeight || 128],
      u_borderWidth: borderWidth,
      u_borderColor: (0, import_l7_utils.rgb2arr)(borderColor),
      u_linearDir: linearDir === import_interface.LinearDir.VERTICAL ? 1 : 0,
      u_linearColor: useLinearColor,
      u_sourceColor: sourceColorArr,
      u_targetColor: targetColorArr,
      u_heightfixed: Number(heightfixed),
      u_vertexScale: vertexHeightScale,
      u_raisingHeight: Number(raisingHeight),
      u_arrow: Number(arrow.enable),
      u_arrowHeight: arrow.arrowHeight || 3,
      u_arrowWidth: arrow.arrowWidth || 2,
      u_tailWidth: arrow.tailWidth === void 0 ? 1 : arrow.tailWidth
    };
  }
  getAnimateUniforms() {
    const { animateOption } = this.layer.getLayerConfig();
    return {
      u_aimate: this.animateOption2Array(animateOption),
      u_time: this.layer.getLayerAnimateTime()
    };
  }
  initModels(callbackModel) {
    this.updateTexture();
    this.iconService.on("imageUpdate", this.updateTexture);
    this.buildModels(callbackModel);
  }
  clearModels() {
    var _a, _b;
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.dataTexture) == null ? void 0 : _b.destroy();
    this.iconService.off("imageUpdate", this.updateTexture);
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      depth = false,
      workerEnabled = false,
      enablePicking
    } = this.layer.getLayerConfig();
    const { frag, vert, type } = this.getShaders();
    this.layer.triangulation = import_l7_utils.LineTriangulation;
    this.layer.buildLayerModel({
      moduleName: "line" + type,
      vertexShader: vert,
      fragmentShader: frag,
      triangulation: import_l7_utils.LineTriangulation,
      depth: { enable: depth },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside),
      workerEnabled,
      workerOptions: {
        modelType: "line" + type,
        enablePicking,
        iconMap: this.iconService.getIconMap()
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  getShaders() {
    const {
      sourceColor,
      targetColor,
      lineType
    } = this.layer.getLayerConfig();
    if (lineType === "dash") {
      return {
        frag: import_line_dash_frag.default,
        vert: import_line_dash_vert.default,
        type: "Dash"
      };
    }
    if (sourceColor && targetColor) {
      return {
        frag: import_line_linear_frag.default,
        vert: import_line_vert.default,
        type: "Linear"
      };
    } else {
      return {
        frag: import_line_frag.default,
        vert: import_line_vert.default,
        type: ""
      };
    }
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "distanceAndIndex",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_DistanceAndIndex",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx, normal, vertexIndex) => {
          return vertexIndex === void 0 ? [vertex[3], 10] : [vertex[3], vertexIndex];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "total_distance",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Total_Distance",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[5]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 1 } = feature;
          return Array.isArray(size) ? [size[0], size[1]] : [size, 0];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "miter",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Miter",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[4]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_iconMapUV",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const iconMap = this.iconService.getIconMap();
          const { texture } = feature;
          const { x, y } = iconMap[texture] || { x: 0, y: 0 };
          return [x, y];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
