var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/line/models/linearline.ts
var linearline_exports = {};
__export(linearline_exports, {
  default: () => LinearLineModel
});
module.exports = __toCommonJS(linearline_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_interface = require("../../core/interface");
var import_triangulation = require("../../core/triangulation");
var import_line_linear_frag = __toESM(require("../shaders/linearLine/line_linear_frag.glsl"));
var import_line_linear_vert = __toESM(require("../shaders/linearLine/line_linear_vert.glsl"));
var LinearLineModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.updateTexture = () => {
      const { createTexture2D } = this.rendererService;
      if (this.colorTexture) {
        this.colorTexture.destroy();
      }
      const {
        rampColors
      } = this.layer.getLayerConfig();
      const imageData = (0, import_l7_utils.generateColorRamp)(rampColors);
      this.colorTexture = createTexture2D({
        data: new Uint8Array(imageData.data),
        width: imageData.width,
        height: imageData.height,
        wrapS: import_l7_core.gl.CLAMP_TO_EDGE,
        wrapT: import_l7_core.gl.CLAMP_TO_EDGE,
        min: import_l7_core.gl.NEAREST,
        mag: import_l7_core.gl.NEAREST,
        flipY: false
      });
    };
  }
  getUninforms() {
    const {
      opacity = 1,
      vertexHeightScale = 20,
      raisingHeight = 0,
      heightfixed = false,
      linearDir = import_interface.LinearDir.VERTICAL
    } = this.layer.getLayerConfig();
    if (this.rendererService.getDirty()) {
      this.colorTexture.bind();
    }
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_linearDir: linearDir === import_interface.LinearDir.VERTICAL ? 1 : 0,
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_texture: this.colorTexture,
      u_heightfixed: Number(heightfixed),
      u_vertexScale: vertexHeightScale,
      u_raisingHeight: Number(raisingHeight)
    };
  }
  initModels(callbackModel) {
    this.updateTexture();
    this.buildModels(callbackModel);
  }
  clearModels() {
    var _a, _b;
    (_a = this.colorTexture) == null ? void 0 : _a.destroy();
    (_b = this.dataTexture) == null ? void 0 : _b.destroy();
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      depth = false
    } = this.layer.getLayerConfig();
    this.layer.triangulation = import_triangulation.LineTriangulation;
    this.layer.buildLayerModel({
      moduleName: "lineRampColors",
      vertexShader: import_line_linear_vert.default,
      fragmentShader: import_line_linear_frag.default,
      triangulation: import_triangulation.LineTriangulation,
      depth: { enable: depth },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "distanceAndIndex",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_DistanceAndIndex",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx, normal, vertexIndex) => {
          return vertexIndex === void 0 ? [vertex[3], 10] : [vertex[3], vertexIndex];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "total_distance",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Total_Distance",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[5]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 1 } = feature;
          return Array.isArray(size) ? [size[0], size[1]] : [size, 0];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "miter",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Miter",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[4]];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
