var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/line/models/simpleLine.ts
var simpleLine_exports = {};
__export(simpleLine_exports, {
  default: () => SimpleLineModel
});
module.exports = __toCommonJS(simpleLine_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_simpleline_frag = __toESM(require("../shaders/simple/simpleline_frag.glsl"));
var import_simpleline_linear_frag = __toESM(require("../shaders/simple/simpleline_linear_frag.glsl"));
var import_simpleline_vert = __toESM(require("../shaders/simple/simpleline_vert.glsl"));
var SimpleLineModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      sourceColor,
      targetColor,
      vertexHeightScale = 20
    } = this.layer.getLayerConfig();
    let useLinearColor = 0;
    let sourceColorArr = [0, 0, 0, 0];
    let targetColorArr = [0, 0, 0, 0];
    if (sourceColor && targetColor) {
      sourceColorArr = (0, import_l7_utils.rgb2arr)(sourceColor);
      targetColorArr = (0, import_l7_utils.rgb2arr)(targetColor);
      useLinearColor = 1;
    }
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_linearColor: useLinearColor,
      u_sourceColor: sourceColorArr,
      u_targetColor: targetColorArr,
      u_vertexScale: vertexHeightScale
    };
  }
  initModels(callbackModel) {
    this.buildModels(callbackModel);
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  getShaders() {
    const {
      sourceColor,
      targetColor
    } = this.layer.getLayerConfig();
    if (sourceColor && targetColor) {
      return {
        frag: import_simpleline_linear_frag.default,
        vert: import_simpleline_vert.default,
        type: "lineSimpleLinear"
      };
    } else {
      return {
        frag: import_simpleline_frag.default,
        vert: import_simpleline_vert.default,
        type: "lineSimpleNormal"
      };
    }
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    const { frag, vert, type } = this.getShaders();
    this.layer.buildLayerModel({
      moduleName: type,
      vertexShader: vert,
      fragmentShader: frag,
      triangulation: import_triangulation.SimpleLineTriangulation,
      primitive: import_l7_core.gl.LINES,
      depth: { enable: false },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "distance",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Distance",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "total_distance",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Total_Distance",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[5]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 1 } = feature;
          return Array.isArray(size) ? [size[0], size[1]] : [size, 0];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
