var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};

// src/plugins/DataMappingPlugin.ts
var DataMappingPlugin_exports = {};
__export(DataMappingPlugin_exports, {
  default: () => DataMappingPlugin
});
module.exports = __toCommonJS(DataMappingPlugin_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_maps = require("@antv/l7-maps");
var import_l7_utils = require("@antv/l7-utils");
var import_inversify = require("inversify");
var import_lodash = require("lodash");
var import_reflect_metadata = require("reflect-metadata");
var DataMappingPlugin = class {
  apply(layer, {
    styleAttributeService
  }) {
    layer.hooks.init.tap("DataMappingPlugin", () => {
      const source = layer.getSource();
      if (source.inited) {
        this.generateMaping(layer, { styleAttributeService });
      } else {
        source.once("sourceUpdate", () => {
          this.generateMaping(layer, { styleAttributeService });
        });
      }
    });
    layer.hooks.beforeRenderData.tap("DataMappingPlugin", () => {
      layer.dataState.dataMappingNeedUpdate = false;
      const source = layer.getSource();
      if (source.inited) {
        this.generateMaping(layer, { styleAttributeService });
      } else {
        source.once("sourceUpdate", () => {
          this.generateMaping(layer, { styleAttributeService });
        });
      }
      return true;
    });
    layer.hooks.beforeRender.tap("DataMappingPlugin", () => {
      const source = layer.getSource();
      if (layer.layerModelNeedUpdate || !source || !source.inited) {
        return;
      }
      const bottomColor = layer.getBottomColor();
      const attributes = styleAttributeService.getLayerStyleAttributes() || [];
      const filter = styleAttributeService.getLayerStyleAttribute("filter");
      const { dataArray } = source.data;
      const attributesToRemapping = attributes.filter((attribute) => attribute.needRemapping);
      let filterData = dataArray;
      if ((filter == null ? void 0 : filter.needRemapping) && (filter == null ? void 0 : filter.scale)) {
        filterData = dataArray.filter((record) => {
          return this.applyAttributeMapping(filter, record, bottomColor)[0];
        });
      }
      if (attributesToRemapping.length) {
        if (filter == null ? void 0 : filter.needRemapping) {
          layer.setEncodedData(this.mapping(layer, attributes, filterData, void 0, bottomColor));
          filter.needRemapping = false;
        } else {
          layer.setEncodedData(this.mapping(layer, attributesToRemapping, filterData, layer.getEncodedData(), bottomColor));
        }
        layer.emit("remapping", null);
      }
    });
  }
  generateMaping(layer, {
    styleAttributeService
  }) {
    const bottomColor = layer.getBottomColor();
    const attributes = styleAttributeService.getLayerStyleAttributes() || [];
    const filter = styleAttributeService.getLayerStyleAttribute("filter");
    const { dataArray } = layer.getSource().data;
    let filterData = dataArray;
    if (filter == null ? void 0 : filter.scale) {
      filterData = dataArray.filter((record) => {
        return this.applyAttributeMapping(filter, record, bottomColor)[0];
      });
    }
    layer.setEncodedData(this.mapping(layer, attributes, filterData, void 0, bottomColor));
    layer.emit("dataUpdate", null);
  }
  getArrowPoints(p1, p2) {
    const dir = [p2[0] - p1[0], p2[1] - p1[1]];
    const normalizeDir = (0, import_l7_utils.normalize)(dir);
    const arrowPoint = [
      p1[0] + normalizeDir[0] * 1e-4,
      p1[1] + normalizeDir[1] * 1e-4
    ];
    return arrowPoint;
  }
  mapping(layer, attributes, data, predata, minimumColor) {
    const {
      arrow = {
        enable: false
      }
    } = layer.getLayerConfig();
    const mappedData = data.map((record, i) => {
      const preRecord = predata ? predata[i] : {};
      const encodeRecord = {
        id: record._id,
        coordinates: record.coordinates,
        ...preRecord
      };
      attributes.filter((attribute) => attribute.scale !== void 0).forEach((attribute) => {
        let values = this.applyAttributeMapping(attribute, record, minimumColor);
        attribute.needRemapping = false;
        if (attribute.name === "color") {
          values = values.map((c) => {
            return (0, import_l7_utils.rgb2arr)(c);
          });
        }
        encodeRecord[attribute.name] = Array.isArray(values) && values.length === 1 ? values[0] : values;
        if (attribute.name === "shape") {
          encodeRecord.shape = this.fontService.getIconFontKey(encodeRecord[attribute.name]);
        }
      });
      if (arrow.enable && (encodeRecord.shape === "line" || encodeRecord.shape === "halfLine")) {
        const coords = encodeRecord.coordinates;
        const arrowPoint = this.getArrowPoints(coords[0], coords[1]);
        encodeRecord.coordinates.splice(1, 0, arrowPoint, arrowPoint);
      }
      return encodeRecord;
    });
    this.adjustData2Amap2Coordinates(mappedData, layer);
    this.adjustData2SimpleCoordinates(mappedData);
    return mappedData;
  }
  adjustData2Amap2Coordinates(mappedData, layer) {
    if (mappedData.length > 0 && this.mapService.version === import_l7_maps.Version["GAODE2.x"]) {
      const layerCenter = this.getLayerCenter(layer);
      if (typeof mappedData[0].coordinates[0] === "number") {
        mappedData.filter((d) => !d.originCoordinates).map((d) => {
          d.version = import_l7_maps.Version["GAODE2.x"];
          d.originCoordinates = (0, import_lodash.cloneDeep)(d.coordinates);
          d.coordinates = this.mapService.lngLatToCoordByLayer(d.coordinates, layerCenter);
        });
      } else {
        mappedData.filter((d) => !d.originCoordinates).map((d) => {
          d.version = import_l7_maps.Version["GAODE2.x"];
          d.originCoordinates = (0, import_lodash.cloneDeep)(d.coordinates);
          d.coordinates = this.mapService.lngLatToCoordsByLayer(d.coordinates, layerCenter);
        });
      }
    }
  }
  getLayerCenter(layer) {
    const source = layer.getSource();
    return source.center;
  }
  adjustData2SimpleCoordinates(mappedData) {
    if (mappedData.length > 0 && this.mapService.version === import_l7_maps.Version.SIMPLE) {
      mappedData.map((d) => {
        if (!d.simpleCoordinate) {
          d.coordinates = this.unProjectCoordinates(d.coordinates);
          d.simpleCoordinate = true;
        }
      });
    }
  }
  unProjectCoordinates(coordinates) {
    if (typeof coordinates[0] === "number") {
      return this.mapService.simpleMapCoord.unproject(coordinates);
    }
    if (coordinates[0] && coordinates[0][0] instanceof Array) {
      const coords = [];
      coordinates.map((coord) => {
        const c1 = [];
        coord.map((co) => {
          c1.push(this.mapService.simpleMapCoord.unproject(co));
        });
        coords.push(c1);
      });
      return coords;
    } else {
      const coords = [];
      coordinates.map((coord) => {
        coords.push(this.mapService.simpleMapCoord.unproject(coord));
      });
      return coords;
    }
  }
  applyAttributeMapping(attribute, record, minimumColor) {
    var _a;
    if (!attribute.scale) {
      return [];
    }
    const scalers = ((_a = attribute == null ? void 0 : attribute.scale) == null ? void 0 : _a.scalers) || [];
    const params = [];
    scalers.forEach(({ field }) => {
      var _a2;
      if (record.hasOwnProperty(field) || ((_a2 = attribute.scale) == null ? void 0 : _a2.type) === "variable") {
        params.push(record[field]);
      }
    });
    const mappingResult = attribute.mapping ? attribute.mapping(params) : [];
    if (attribute.name === "color" && !(0, import_l7_utils.isColor)(mappingResult[0])) {
      return [minimumColor];
    }
    return mappingResult;
  }
};
__decorateClass([
  (0, import_inversify.inject)(import_l7_core.TYPES.IGlobalConfigService)
], DataMappingPlugin.prototype, "configService", 2);
__decorateClass([
  (0, import_inversify.inject)(import_l7_core.TYPES.IMapService)
], DataMappingPlugin.prototype, "mapService", 2);
__decorateClass([
  (0, import_inversify.inject)(import_l7_core.TYPES.IFontService)
], DataMappingPlugin.prototype, "fontService", 2);
DataMappingPlugin = __decorateClass([
  (0, import_inversify.injectable)()
], DataMappingPlugin);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
