var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/index.ts
var point_exports = {};
__export(point_exports, {
  default: () => PointLayer
});
module.exports = __toCommonJS(point_exports);
var import_BaseLayer = __toESM(require("../core/BaseLayer"));
var import_models = __toESM(require("./models/index"));
var PointLayer = class extends import_BaseLayer.default {
  constructor() {
    super(...arguments);
    this.type = "PointLayer";
  }
  buildModels() {
    const modelType = this.getModelType();
    this.layerModel = new import_models.default[modelType](this);
    this.layerModel.initModels((models) => {
      this.models = models;
      this.layerService.updateLayerRenderList();
      this.renderLayers();
    });
  }
  rebuildModels() {
    this.layerModel.buildModels((models) => this.models = models);
  }
  getModelTypeWillEmptyData() {
    if (this.shapeOption) {
      const { field, values } = this.shapeOption;
      const { shape2d, shape3d } = this.getLayerConfig();
      const iconMap = this.iconService.getIconMap();
      if (field && (shape2d == null ? void 0 : shape2d.indexOf(field)) !== -1) {
        return "fill";
      }
      if (values && values instanceof Array) {
        for (const v of values) {
          if (typeof v === "string" && iconMap.hasOwnProperty(v)) {
            return "image";
          }
        }
      }
    }
    return "normal";
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  getDefaultConfig() {
    const type = this.getModelType();
    const defaultConfig = {
      fillImage: {},
      normal: {
        blend: "additive"
      },
      radar: {},
      simplePoint: {},
      fill: { blend: "normal" },
      extrude: {},
      image: {},
      text: {
        blend: "normal"
      },
      vectorpoint: {},
      tile: {},
      earthFill: {},
      earthExtrude: {}
    };
    return defaultConfig[type];
  }
  getModelType() {
    const PointTypes = [
      "fillImage",
      "fill",
      "radar",
      "image",
      "normal",
      "simplePoint",
      "extrude",
      "text",
      "vectorpoint",
      "tile",
      "earthFill",
      "earthExtrude"
    ];
    if (this.layerSource.parser.type === "mvt") {
      return "vectorpoint";
    }
    const layerData = this.getEncodedData();
    const { shape2d, shape3d } = this.getLayerConfig();
    const iconMap = this.iconService.getIconMap();
    const item = layerData.find((fe) => {
      return fe.hasOwnProperty("shape");
    });
    if (!item) {
      return this.getModelTypeWillEmptyData();
    } else {
      const shape = item.shape;
      if (shape === "dot") {
        return "normal";
      }
      if (shape === "simple") {
        return "simplePoint";
      }
      if (shape === "radar") {
        return "radar";
      }
      if (this.layerType === "fillImage") {
        return "fillImage";
      }
      if ((shape2d == null ? void 0 : shape2d.indexOf(shape)) !== -1) {
        if (this.mapService.version === "GLOBEL") {
          return "earthFill";
        } else {
          return "fill";
        }
      }
      if ((shape3d == null ? void 0 : shape3d.indexOf(shape)) !== -1) {
        if (this.mapService.version === "GLOBEL") {
          return "earthExtrude";
        } else {
          return "extrude";
        }
      }
      if (iconMap.hasOwnProperty(shape)) {
        return "image";
      }
      return "text";
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
