var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/models/earthExtrude.ts
var earthExtrude_exports = {};
__export(earthExtrude_exports, {
  default: () => ExtrudeModel
});
module.exports = __toCommonJS(earthExtrude_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_utils = require("../../earth/utils");
var import_extrude_frag = __toESM(require("../shaders/earth/extrude_frag.glsl"));
var import_extrude_vert = __toESM(require("../shaders/earth/extrude_vert.glsl"));
var ExtrudeModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.raiseCount = 0;
    this.raiserepeat = 0;
  }
  getUninforms() {
    const {
      animateOption = {
        enable: false,
        speed: 0.01,
        repeat: false
      },
      opacity = 1,
      sourceColor,
      targetColor,
      pickLight = false,
      heightfixed = false,
      opacityLinear = {
        enable: false,
        dir: "up"
      },
      lightEnable = true
    } = this.layer.getLayerConfig();
    if (this.dataTextureTest && this.dataTextureNeedUpdate({
      opacity
    })) {
      this.judgeStyleAttributes({
        opacity
      });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    let useLinearColor = 0;
    let sourceColorArr = [0, 0, 0, 0];
    let targetColorArr = [0, 0, 0, 0];
    if (sourceColor && targetColor) {
      sourceColorArr = (0, import_l7_utils.rgb2arr)(sourceColor);
      targetColorArr = (0, import_l7_utils.rgb2arr)(targetColor);
      useLinearColor = 1;
    }
    if (this.raiseCount < 1 && this.raiserepeat > 0) {
      if (animateOption.enable) {
        const { speed = 0.01, repeat = false } = animateOption;
        this.raiseCount += speed;
        if (this.raiseCount >= 1) {
          if (this.raiserepeat > 1) {
            this.raiseCount = 0;
            this.raiserepeat--;
          } else {
            this.raiseCount = 1;
          }
        }
      }
    }
    return {
      u_pickLight: Number(pickLight),
      u_heightfixed: Number(heightfixed),
      u_r: animateOption.enable && this.raiserepeat > 0 ? this.raiseCount : 1,
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_linearColor: useLinearColor,
      u_sourceColor: sourceColorArr,
      u_targetColor: targetColorArr,
      u_opacitylinear: Number(opacityLinear.enable),
      u_opacitylinear_dir: opacityLinear.dir === "up" ? 1 : 0,
      u_lightEnable: Number(lightEnable)
    };
  }
  initModels(callbackModel) {
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    const {
      animateOption: { repeat = 1 }
    } = this.layer.getLayerConfig();
    this.raiserepeat = repeat;
    this.layer.buildLayerModel({
      moduleName: "pointEarthExtrude",
      vertexShader: import_extrude_vert.default,
      fragmentShader: import_extrude_frag.default,
      triangulation: import_triangulation.PointExtrudeTriangulation,
      depth: { enable: true },
      cull: {
        enable: true,
        face: (0, import_l7_utils.getCullFace)(this.mapService.version)
      },
      blend: this.getBlend()
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size } = feature;
          if (size) {
            let buffersize = [];
            if (Array.isArray(size)) {
              buffersize = size.length === 2 ? [size[0], size[0], size[1]] : size;
            }
            if (!Array.isArray(size)) {
              buffersize = [size, size, size];
            }
            return buffersize;
          } else {
            return [2, 2, 2];
          }
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "pos",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Pos",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx) => {
          const coordinates = (0, import_l7_utils.calculateCentroid)(feature.coordinates);
          return (0, import_utils.lglt2xyz)([coordinates[0], coordinates[1]]);
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
