var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/models/earthFill.ts
var earthFill_exports = {};
__export(earthFill_exports, {
  default: () => FillModel
});
module.exports = __toCommonJS(earthFill_exports);
var import_l7_core = require("@antv/l7-core");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_fill_frag = __toESM(require("../shaders/earth/fill_frag.glsl"));
var import_fill_vert = __toESM(require("../shaders/earth/fill_vert.glsl"));
var import_gl_matrix = require("gl-matrix");
var FillModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      strokeOpacity = 1,
      strokeWidth = 0,
      stroke = "rgba(0,0,0,0)",
      offsets = [0, 0],
      blend,
      blur = 0
    } = this.layer.getLayerConfig();
    if (this.dataTextureTest && this.dataTextureNeedUpdate({
      opacity,
      strokeOpacity,
      strokeWidth,
      stroke,
      offsets
    })) {
      this.judgeStyleAttributes({
        opacity,
        strokeOpacity,
        strokeWidth,
        stroke,
        offsets
      });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_blur: blur,
      u_additive: blend === "additive" ? 1 : 0,
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_stroke_opacity: (0, import_lodash.isNumber)(strokeOpacity) ? strokeOpacity : 1,
      u_stroke_width: (0, import_lodash.isNumber)(strokeWidth) ? strokeWidth : 1,
      u_stroke_color: this.getStrokeColor(stroke),
      u_offsets: this.isOffsetStatic(offsets) ? offsets : [0, 0]
    };
  }
  getAnimateUniforms() {
    const {
      animateOption = { enable: false }
    } = this.layer.getLayerConfig();
    return {
      u_aimate: this.animateOption2Array(animateOption),
      u_time: this.layer.getLayerAnimateTime()
    };
  }
  initModels(callbackModel) {
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    this.layer.triangulation = import_triangulation.GlobelPointFillTriangulation;
    this.layer.buildLayerModel({
      moduleName: "pointEarthFill",
      vertexShader: import_fill_vert.default,
      fragmentShader: import_fill_frag.default,
      triangulation: import_triangulation.GlobelPointFillTriangulation,
      depth: { enable: true },
      blend: this.getBlend()
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  animateOption2Array(option) {
    return [option.enable ? 0 : 1, option.speed || 1, option.rings || 3, 0];
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "extrude",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Extrude",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const [x, y, z] = vertex;
          const n1 = import_gl_matrix.vec3.fromValues(0, 0, 1);
          const n2 = import_gl_matrix.vec3.fromValues(x, 0, z);
          const xzReg = x >= 0 ? import_gl_matrix.vec3.angle(n1, n2) : Math.PI * 2 - import_gl_matrix.vec3.angle(n1, n2);
          const yReg = Math.PI * 2 - Math.asin(y / 100);
          const m = import_gl_matrix.mat4.create();
          import_gl_matrix.mat4.rotateY(m, m, xzReg);
          import_gl_matrix.mat4.rotateX(m, m, yReg);
          const v1 = import_gl_matrix.vec3.fromValues(1, 1, 0);
          import_gl_matrix.vec3.transformMat4(v1, v1, m);
          import_gl_matrix.vec3.normalize(v1, v1);
          const v2 = import_gl_matrix.vec3.fromValues(-1, 1, 0);
          import_gl_matrix.vec3.transformMat4(v2, v2, m);
          import_gl_matrix.vec3.normalize(v2, v2);
          const v3 = import_gl_matrix.vec3.fromValues(-1, -1, 0);
          import_gl_matrix.vec3.transformMat4(v3, v3, m);
          import_gl_matrix.vec3.normalize(v3, v3);
          const v4 = import_gl_matrix.vec3.fromValues(1, -1, 0);
          import_gl_matrix.vec3.transformMat4(v4, v4, m);
          import_gl_matrix.vec3.normalize(v4, v4);
          const extrude = [...v1, ...v2, ...v3, ...v4];
          const extrudeIndex = attributeIdx % 4 * 3;
          return [
            extrude[extrudeIndex],
            extrude[extrudeIndex + 1],
            extrude[extrudeIndex + 2]
          ];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 5 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "shape",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Shape",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { shape = 2 } = feature;
          const shape2d = this.layer.getLayerConfig().shape2d;
          const shapeIndex = shape2d.indexOf(shape);
          return [shapeIndex];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
