var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/models/fill.ts
var fill_exports = {};
__export(fill_exports, {
  default: () => FillModel
});
module.exports = __toCommonJS(fill_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_wave_frag = __toESM(require("../shaders/animate/wave_frag.glsl"));
var import_fill_frag = __toESM(require("../shaders/fill_frag.glsl"));
var import_fill_vert = __toESM(require("../shaders/fill_vert.glsl"));
var import_l7_maps = require("@antv/l7-maps");
var FillModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.meter2coord = 1;
    this.meteryScale = 1;
    this.isMeter = false;
    this.unit = "l7size";
  }
  getUninforms() {
    const {
      opacity = 1,
      strokeOpacity = 1,
      strokeWidth = 0,
      stroke = "rgba(0,0,0,0)",
      offsets = [0, 0],
      blend,
      blur = 0,
      raisingHeight = 0,
      unit = "l7size"
    } = this.layer.getLayerConfig();
    this.updateUnit(unit);
    if (this.dataTextureTest && this.dataTextureNeedUpdate({
      opacity,
      strokeOpacity,
      strokeWidth,
      stroke,
      offsets
    })) {
      this.judgeStyleAttributes({
        opacity,
        strokeOpacity,
        strokeWidth,
        stroke,
        offsets
      });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_raisingHeight: Number(raisingHeight),
      u_meter2coord: this.meter2coord,
      u_meteryScale: this.meteryScale,
      u_isMeter: Number(this.isMeter),
      u_blur: blur,
      u_additive: blend === "additive" ? 1 : 0,
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_stroke_opacity: (0, import_lodash.isNumber)(strokeOpacity) ? strokeOpacity : 1,
      u_stroke_width: (0, import_lodash.isNumber)(strokeWidth) ? strokeWidth : 1,
      u_stroke_color: this.getStrokeColor(stroke),
      u_offsets: this.isOffsetStatic(offsets) ? offsets : [0, 0]
    };
  }
  getAnimateUniforms() {
    const {
      animateOption = { enable: false }
    } = this.layer.getLayerConfig();
    return {
      u_aimate: this.animateOption2Array(animateOption),
      u_time: this.layer.getLayerAnimateTime()
    };
  }
  getAttribute() {
    return this.styleAttributeService.createAttributesAndIndices(this.layer.getEncodedData(), import_l7_utils.PointFillTriangulation);
  }
  initModels(callbackModel) {
    this.updateUnit("l7size");
    this.buildModels(callbackModel);
  }
  calMeter2Coord() {
    var _a;
    const [minLng, minLat, maxLng, maxLat] = this.layer.getSource().extent;
    const center = [(minLng + maxLng) / 2, (minLat + maxLat) / 2];
    const { version } = this.mapService;
    const mapboxContext = (_a = import_l7_utils.$window) == null ? void 0 : _a.mapboxgl;
    if (version === import_l7_maps.Version.MAPBOX && (mapboxContext == null ? void 0 : mapboxContext.MercatorCoordinate)) {
      const coord = mapboxContext.MercatorCoordinate.fromLngLat({ lng: center[0], lat: center[1] }, 0);
      const offsetInMercatorCoordinateUnits = coord.meterInMercatorCoordinateUnits();
      const westCoord = new mapboxContext.MercatorCoordinate(coord.x - offsetInMercatorCoordinateUnits, coord.y, coord.z);
      const westLnglat = westCoord.toLngLat();
      const southCoord = new mapboxContext.MercatorCoordinate(coord.x, coord.y - offsetInMercatorCoordinateUnits, coord.z);
      const southLnglat = southCoord.toLngLat();
      this.meter2coord = center[0] - westLnglat.lng;
      this.meteryScale = (southLnglat.lat - center[1]) / this.meter2coord;
      return;
    }
    const m1 = this.mapService.meterToCoord(center, [minLng, minLat]);
    const m2 = this.mapService.meterToCoord(center, [
      maxLng === minLng ? maxLng + 0.1 : maxLng,
      maxLat === minLat ? minLat + 0.1 : maxLat
    ]);
    this.meter2coord = (m1 + m2) / 2;
    if (!Boolean(this.meter2coord)) {
      this.meter2coord = 7.70681090738883;
    }
  }
  async buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      animateOption = { enable: false },
      workerEnabled = false,
      enablePicking,
      shape2d
    } = this.layer.getLayerConfig();
    const { frag, vert, type } = this.getShaders(animateOption);
    this.layer.triangulation = import_l7_utils.PointFillTriangulation;
    this.layer.buildLayerModel({
      moduleName: type,
      vertexShader: vert,
      fragmentShader: frag,
      triangulation: import_l7_utils.PointFillTriangulation,
      depth: { enable: false },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside),
      workerEnabled,
      workerOptions: {
        modelType: type,
        enablePicking,
        shape2d
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  getShaders(animateOption) {
    if (animateOption.enable) {
      switch (animateOption.type) {
        case "wave":
          return {
            frag: import_wave_frag.default,
            vert: import_fill_vert.default,
            type: "pointWave"
          };
        default:
          return {
            frag: import_wave_frag.default,
            vert: import_fill_vert.default,
            type: "pointWave"
          };
      }
    } else {
      return {
        frag: import_fill_frag.default,
        vert: import_fill_vert.default,
        type: "pointFill"
      };
    }
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  animateOption2Array(option) {
    return [option.enable ? 0 : 1, option.speed || 1, option.rings || 3, 0];
  }
  registerBuiltinAttributes() {
    const shape2d = this.layer.getLayerConfig().shape2d;
    this.styleAttributeService.registerStyleAttribute({
      name: "extrude",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Extrude",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const extrude = [1, 1, 0, -1, 1, 0, -1, -1, 0, 1, -1, 0];
          const extrudeIndex = attributeIdx % 4 * 3;
          return [
            extrude[extrudeIndex],
            extrude[extrudeIndex + 1],
            extrude[extrudeIndex + 2]
          ];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 5 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "shape",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Shape",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { shape = 2 } = feature;
          const shapeIndex = shape2d.indexOf(shape);
          return [shapeIndex];
        }
      }
    });
  }
  updateUnit(unit) {
    const { version } = this.mapService;
    if (this.unit !== unit) {
      if (this.unit !== "meter" && unit === "meter" && version !== import_l7_maps.Version.L7MAP && version !== import_l7_maps.Version.GLOBEL) {
        this.isMeter = true;
        this.calMeter2Coord();
      } else if (this.unit === "meter" && unit !== "meter") {
        this.isMeter = false;
        this.meter2coord = 1;
      }
      this.unit = unit;
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
