var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/models/fillmage.ts
var fillmage_exports = {};
__export(fillmage_exports, {
  default: () => FillImageModel
});
module.exports = __toCommonJS(fillmage_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_maps = require("@antv/l7-maps");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_fillImage_frag = __toESM(require("../shaders/image/fillImage_frag.glsl"));
var import_fillImage_vert = __toESM(require("../shaders/image/fillImage_vert.glsl"));
var FillImageModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.meter2coord = 1;
    this.isMeter = false;
    this.radian = 0;
    this.updateTexture = () => {
      const { createTexture2D } = this.rendererService;
      if (this.texture) {
        this.texture.update({
          data: this.iconService.getCanvas(),
          mag: "linear",
          min: "linear mipmap nearest",
          mipmap: true
        });
        this.layer.renderLayers();
        return;
      }
      this.texture = createTexture2D({
        data: this.iconService.getCanvas(),
        mag: import_l7_core.gl.LINEAR,
        min: import_l7_core.gl.LINEAR_MIPMAP_LINEAR,
        premultiplyAlpha: false,
        width: 1024,
        height: this.iconService.canvasHeight || 128,
        mipmap: true
      });
    };
  }
  getUninforms() {
    const {
      opacity = 1,
      offsets = [0, 0],
      rotation
    } = this.layer.getLayerConfig();
    if (this.rendererService.getDirty()) {
      this.texture.bind();
    }
    let rotateFlag = 1;
    if (this.mapService.version === "GAODE2.x" || this.mapService.version === "GAODE1.x") {
      rotateFlag = -1;
    }
    this.radian = rotation !== void 0 ? rotateFlag * Math.PI * rotation / 180 : rotateFlag * Math.PI * (this.mapService.getRotation() % 360) / 180;
    if (this.dataTextureTest && this.dataTextureNeedUpdate({
      opacity,
      offsets
    })) {
      this.judgeStyleAttributes({
        opacity,
        offsets
      });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_isMeter: Number(this.isMeter),
      u_RotateMatrix: new Float32Array([
        Math.cos(this.radian),
        Math.sin(this.radian),
        -Math.sin(this.radian),
        Math.cos(this.radian)
      ]),
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_texture: this.texture,
      u_textSize: [1024, this.iconService.canvasHeight || 128],
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_offsets: this.isOffsetStatic(offsets) ? offsets : [0, 0]
    };
  }
  getAttribute() {
    return this.styleAttributeService.createAttributesAndIndices(this.layer.getEncodedData(), import_triangulation.PointFillTriangulation);
  }
  initModels(callbackModel) {
    this.updateTexture();
    this.iconService.on("imageUpdate", this.updateTexture);
    const {
      unit = "l7size"
    } = this.layer.getLayerConfig();
    const { version } = this.mapService;
    if (unit === "meter" && version !== import_l7_maps.Version.L7MAP && version !== import_l7_maps.Version.GLOBEL) {
      this.isMeter = true;
      this.calMeter2Coord();
    }
    this.buildModels(callbackModel);
  }
  calMeter2Coord() {
    const [minLng, minLat, maxLng, maxLat] = this.layer.getSource().extent;
    const center = [(minLng + maxLng) / 2, (minLat + maxLat) / 2];
    const { version } = this.mapService;
    if (version === import_l7_maps.Version.MAPBOX && window.mapboxgl.MercatorCoordinate) {
      const coord = window.mapboxgl.MercatorCoordinate.fromLngLat({ lng: center[0], lat: center[1] }, 0);
      const offsetInMeters = 1;
      const offsetInMercatorCoordinateUnits = offsetInMeters * coord.meterInMercatorCoordinateUnits();
      const westCoord = new window.mapboxgl.MercatorCoordinate(coord.x - offsetInMercatorCoordinateUnits, coord.y, coord.z);
      const westLnglat = westCoord.toLngLat();
      this.meter2coord = center[0] - westLnglat.lng;
      return;
    }
    const m1 = this.mapService.meterToCoord(center, [minLng, minLat]);
    const m2 = this.mapService.meterToCoord(center, [
      maxLng === minLng ? maxLng + 0.1 : maxLng,
      maxLat === minLat ? minLat + 0.1 : maxLat
    ]);
    this.meter2coord = (m1 + m2) / 2;
    if (!Boolean(this.meter2coord)) {
      this.meter2coord = 7.70681090738883;
    }
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    this.layer.buildLayerModel({
      moduleName: "pointFillImage",
      vertexShader: import_fillImage_vert.default,
      fragmentShader: import_fillImage_frag.default,
      triangulation: import_triangulation.PointFillTriangulation,
      depth: { enable: false },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside),
      cull: {
        enable: true,
        face: (0, import_l7_utils.getCullFace)(this.mapService.version)
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  clearModels() {
    var _a, _b;
    this.iconService.off("imageUpdate", this.updateTexture);
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.dataTexture) == null ? void 0 : _b.destroy();
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "rotate",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Rotate",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { rotate = 0 } = feature;
          return Array.isArray(rotate) ? [rotate[0]] : [rotate];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const iconMap = this.iconService.getIconMap();
          const { shape } = feature;
          const { x, y } = iconMap[shape] || { x: 0, y: 0 };
          return [x, y];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "extrude",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Extrude",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const extrude = [1, 1, 0, -1, 1, 0, -1, -1, 0, 1, -1, 0];
          const extrudeIndex = attributeIdx % 4 * 3;
          return [
            extrude[extrudeIndex],
            extrude[extrudeIndex + 1],
            extrude[extrudeIndex + 2]
          ];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 5 } = feature;
          return Array.isArray(size) ? [size[0] * this.meter2coord] : [size * this.meter2coord];
        }
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
