var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/point/models/text.ts
var text_exports = {};
__export(text_exports, {
  TextTriangulation: () => TextTriangulation,
  default: () => TextModel
});
module.exports = __toCommonJS(text_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_collision_index = __toESM(require("../../utils/collision-index"));
var import_symbol_layout = require("../../utils/symbol-layout");
var import_text_frag = __toESM(require("../shaders/text_frag.glsl"));
var import_text_vert = __toESM(require("../shaders/text_vert.glsl"));
function TextTriangulation(feature) {
  const that = this;
  const id = feature.id;
  const vertices = [];
  const indices = [];
  if (!that.glyphInfoMap || !that.glyphInfoMap[id]) {
    return {
      vertices: [],
      indices: [],
      size: 7
    };
  }
  const centroid = that.glyphInfoMap[id].centroid;
  const coord = centroid.length === 2 ? [centroid[0], centroid[1], 0] : centroid;
  that.glyphInfoMap[id].glyphQuads.forEach((quad, index) => {
    vertices.push(...coord, quad.tex.x, quad.tex.y + quad.tex.height, quad.tl.x, quad.tl.y, ...coord, quad.tex.x + quad.tex.width, quad.tex.y + quad.tex.height, quad.tr.x, quad.tr.y, ...coord, quad.tex.x + quad.tex.width, quad.tex.y, quad.br.x, quad.br.y, ...coord, quad.tex.x, quad.tex.y, quad.bl.x, quad.bl.y);
    indices.push(0 + index * 4, 1 + index * 4, 2 + index * 4, 2 + index * 4, 3 + index * 4, 0 + index * 4);
  });
  return {
    vertices,
    indices,
    size: 7
  };
}
var TextModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.glyphInfoMap = {};
    this.currentZoom = -1;
    this.textureHeight = 0;
    this.textCount = 0;
    this.preTextStyle = {};
    this.buildModels = async (callbackModel) => {
      const {
        mask = false,
        maskInside = true
      } = this.layer.getLayerConfig();
      this.mapping();
      this.layer.buildLayerModel({
        moduleName: "pointText",
        vertexShader: import_text_vert.default,
        fragmentShader: import_text_frag.default,
        triangulation: TextTriangulation.bind(this),
        depth: { enable: false },
        blend: this.getBlend(),
        stencil: (0, import_l7_utils.getMask)(mask, maskInside)
      }).then((model) => {
        callbackModel([model]);
      }).catch((err) => {
        console.warn(err);
        callbackModel([]);
      });
    };
    this.mapping = () => {
      this.initGlyph();
      this.updateTexture();
      this.filterGlyphs();
      this.reBuildModel();
    };
  }
  getUninforms() {
    const {
      opacity = 1,
      stroke = "#fff",
      strokeWidth = 0,
      textAnchor = "center",
      textAllowOverlap = false,
      halo = 0.5,
      gamma = 2,
      raisingHeight = 0
    } = this.layer.getLayerConfig();
    const { canvas, mapping } = this.fontService;
    if (Object.keys(mapping).length !== this.textCount) {
      this.updateTexture();
      this.textCount = Object.keys(mapping).length;
    }
    this.preTextStyle = {
      textAnchor,
      textAllowOverlap
    };
    if (this.dataTextureTest && this.dataTextureNeedUpdate({
      opacity,
      strokeWidth,
      stroke
    })) {
      this.judgeStyleAttributes({
        opacity,
        strokeWidth,
        stroke
      });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_raisingHeight: Number(raisingHeight),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_stroke_width: (0, import_lodash.isNumber)(strokeWidth) ? strokeWidth : 1,
      u_stroke_color: this.getStrokeColor(stroke),
      u_sdf_map: this.texture,
      u_halo_blur: halo,
      u_gamma_scale: gamma,
      u_sdf_map_size: [canvas.width, canvas.height]
    };
  }
  initModels(callbackModel) {
    this.layer.on("remapping", this.mapping);
    this.extent = this.textExtent();
    const {
      textAnchor = "center",
      textAllowOverlap = true
    } = this.layer.getLayerConfig();
    this.preTextStyle = {
      textAnchor,
      textAllowOverlap
    };
    this.buildModels(callbackModel);
  }
  needUpdate() {
    const {
      textAllowOverlap = false
    } = this.layer.getLayerConfig();
    const zoom = this.mapService.getZoom();
    const extent = this.mapService.getBounds();
    const flag = (0, import_l7_utils.boundsContains)(this.extent, extent);
    if (!textAllowOverlap && (Math.abs(this.currentZoom - zoom) > 1 || !flag) || textAllowOverlap !== this.preTextStyle.textAllowOverlap) {
      this.reBuildModel();
      return true;
    }
    return false;
  }
  clearModels() {
    var _a, _b;
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.dataTexture) == null ? void 0 : _b.destroy();
    this.layer.off("remapping", this.mapping);
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "rotate",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Rotate",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { rotate = 0 } = feature;
          return Array.isArray(rotate) ? [rotate[0]] : [rotate];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "textOffsets",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_textOffsets",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[5], vertex[6]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 12 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "textUv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_tex",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
  textExtent() {
    const bounds = this.mapService.getBounds();
    return (0, import_l7_utils.padBounds)(bounds, 0.5);
  }
  initTextFont() {
    const {
      fontWeight = "400",
      fontFamily = "sans-serif"
    } = this.layer.getLayerConfig();
    const data = this.layer.getEncodedData();
    const characterSet = [];
    data.forEach((item) => {
      let { shape = "" } = item;
      shape = shape.toString();
      for (const char of shape) {
        if (characterSet.indexOf(char) === -1) {
          characterSet.push(char);
        }
      }
    });
    this.fontService.setFontOptions({
      characterSet,
      fontWeight,
      fontFamily,
      iconfont: false
    });
  }
  initIconFontTex() {
    const {
      fontWeight = "400",
      fontFamily = "sans-serif"
    } = this.layer.getLayerConfig();
    const data = this.layer.getEncodedData();
    const characterSet = [];
    data.forEach((item) => {
      let { shape = "" } = item;
      shape = `${shape}`;
      if (characterSet.indexOf(shape) === -1) {
        characterSet.push(shape);
      }
    });
    this.fontService.setFontOptions({
      characterSet,
      fontWeight,
      fontFamily,
      iconfont: true
    });
  }
  generateGlyphLayout(iconfont) {
    const { mapping } = this.fontService;
    const {
      spacing = 2,
      textAnchor = "center"
    } = this.layer.getLayerConfig();
    const data = this.layer.getEncodedData();
    this.glyphInfo = data.map((feature) => {
      const { shape = "", id, size = 1, textOffset = [0, 0] } = feature;
      const shaping = (0, import_symbol_layout.shapeText)(shape.toString(), mapping, size, textAnchor, "left", spacing, textOffset, iconfont);
      const glyphQuads = (0, import_symbol_layout.getGlyphQuads)(shaping, textOffset, false);
      feature.shaping = shaping;
      feature.glyphQuads = glyphQuads;
      feature.centroid = (0, import_l7_utils.calculateCentroid)(feature.coordinates);
      feature.originCentroid = feature.version === "GAODE2.x" ? (0, import_l7_utils.calculateCentroid)(feature.originCoordinates) : feature.originCentroid = feature.centroid;
      this.glyphInfoMap[id] = {
        shaping,
        glyphQuads,
        centroid: (0, import_l7_utils.calculateCentroid)(feature.coordinates)
      };
      return feature;
    });
  }
  filterGlyphs() {
    const {
      padding = [4, 4],
      textAllowOverlap = false
    } = this.layer.getLayerConfig();
    if (textAllowOverlap) {
      return;
    }
    this.glyphInfoMap = {};
    this.currentZoom = this.mapService.getZoom();
    this.extent = this.textExtent();
    const { width, height } = this.rendererService.getViewportSize();
    const collisionIndex = new import_collision_index.default(width, height);
    const filterData = this.glyphInfo.filter((feature) => {
      const { shaping, id = 0 } = feature;
      const centroid = feature.version === "GAODE2.x" ? feature.originCentroid : feature.centroid;
      const size = feature.size;
      const fontScale = size / 24;
      const pixels = this.mapService.lngLatToContainer(centroid);
      const { box } = collisionIndex.placeCollisionBox({
        x1: shaping.left * fontScale - padding[0],
        x2: shaping.right * fontScale + padding[0],
        y1: shaping.top * fontScale - padding[1],
        y2: shaping.bottom * fontScale + padding[1],
        anchorPointX: pixels.x,
        anchorPointY: pixels.y
      });
      if (box && box.length) {
        collisionIndex.insertCollisionBox(box, id);
        return true;
      } else {
        return false;
      }
    });
    filterData.forEach((item) => {
      this.glyphInfoMap[item.id] = item;
    });
  }
  initGlyph() {
    const { iconfont = false } = this.layer.getLayerConfig();
    iconfont ? this.initIconFontTex() : this.initTextFont();
    this.generateGlyphLayout(iconfont);
  }
  updateTexture() {
    const { createTexture2D } = this.rendererService;
    const { canvas } = this.fontService;
    this.textureHeight = canvas.height;
    if (this.texture) {
      this.texture.destroy();
    }
    this.texture = createTexture2D({
      data: canvas,
      mag: import_l7_core.gl.LINEAR,
      min: import_l7_core.gl.LINEAR,
      width: canvas.width,
      height: canvas.height
    });
  }
  reBuildModel() {
    const {
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    this.filterGlyphs();
    this.layer.buildLayerModel({
      moduleName: "pointText",
      vertexShader: import_text_vert.default,
      fragmentShader: import_text_frag.default,
      triangulation: TextTriangulation.bind(this),
      depth: { enable: false },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      this.layer.models = [model];
      this.layer.renderLayers();
    }).catch((err) => {
      console.warn(err);
      this.layer.models = [];
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  TextTriangulation
});
