var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/polygon/models/extrude.ts
var extrude_exports = {};
__export(extrude_exports, {
  default: () => ExtrudeModel
});
module.exports = __toCommonJS(extrude_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_polygon_extrude_frag = __toESM(require("../shaders/extrude/polygon_extrude_frag.glsl"));
var import_polygon_extrude_vert = __toESM(require("../shaders/extrude/polygon_extrude_vert.glsl"));
var import_polygon_extrudetex_frag = __toESM(require("../shaders/extrude/polygon_extrudetex_frag.glsl"));
var import_polygon_extrudetex_vert = __toESM(require("../shaders/extrude/polygon_extrudetex_vert.glsl"));
var import_polygon_extrude_picklight_frag = __toESM(require("../shaders/extrude/polygon_extrude_picklight_frag.glsl"));
var import_polygon_extrude_picklight_vert = __toESM(require("../shaders/extrude/polygon_extrude_picklight_vert.glsl"));
var ExtrudeModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      heightfixed = false,
      raisingHeight = 0,
      topsurface = true,
      sidesurface = true,
      sourceColor,
      targetColor
    } = this.layer.getLayerConfig();
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    let useLinearColor = 0;
    let sourceColorArr = [1, 1, 1, 1];
    let targetColorArr = [1, 1, 1, 1];
    if (sourceColor && targetColor) {
      sourceColorArr = (0, import_l7_utils.rgb2arr)(sourceColor);
      targetColorArr = (0, import_l7_utils.rgb2arr)(targetColor);
      useLinearColor = 1;
    }
    return {
      u_topsurface: Number(topsurface),
      u_sidesurface: Number(sidesurface),
      u_heightfixed: Number(heightfixed),
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_raisingHeight: Number(raisingHeight),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_linearColor: useLinearColor,
      u_sourceColor: sourceColorArr,
      u_targetColor: targetColorArr,
      u_texture: this.texture
    };
  }
  initModels(callbackModel) {
    this.loadTexture();
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    const { frag, vert, type } = this.getShaders();
    this.layer.buildLayerModel({
      moduleName: type,
      vertexShader: vert,
      fragmentShader: frag,
      triangulation: import_triangulation.PolygonExtrudeTriangulation,
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  getShaders() {
    const {
      pickLight,
      mapTexture
    } = this.layer.getLayerConfig();
    if (mapTexture) {
      return {
        frag: import_polygon_extrudetex_frag.default,
        vert: import_polygon_extrudetex_vert.default,
        type: "polygonExtrudeTexture"
      };
    }
    if (pickLight) {
      return {
        frag: import_polygon_extrude_picklight_frag.default,
        vert: import_polygon_extrude_picklight_vert.default,
        type: "polygonExtrudePickLight"
      };
    } else {
      return {
        frag: import_polygon_extrude_frag.default,
        vert: import_polygon_extrude_vert.default,
        type: "polygonExtrude"
      };
    }
  }
  clearModels() {
    var _a, _b;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
    (_b = this.texture) == null ? void 0 : _b.destroy();
  }
  registerBuiltinAttributes() {
    const bbox = this.layer.getSource().extent;
    const [minLng, minLat, maxLng, maxLat] = bbox;
    const lngLen = maxLng - minLng;
    const latLen = maxLat - minLat;
    this.styleAttributeService.registerStyleAttribute({
      name: "uvs",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_uvs",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          const lng = vertex[0];
          const lat = vertex[1];
          return [(lng - minLng) / lngLen, (lat - minLat) / latLen, vertex[4]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "normal",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Normal",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: "size",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Size",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const { size = 10 } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
  }
  loadTexture() {
    const {
      mapTexture
    } = this.layer.getLayerConfig();
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    if (mapTexture) {
      const image = new Image();
      image.crossOrigin = "";
      image.src = mapTexture;
      image.onload = () => {
        this.texture = createTexture2D({
          data: image,
          width: image.width,
          height: image.height,
          wrapS: import_l7_core.gl.CLAMP_TO_EDGE,
          wrapT: import_l7_core.gl.CLAMP_TO_EDGE,
          min: import_l7_core.gl.LINEAR,
          mag: import_l7_core.gl.LINEAR
        });
        this.layerService.updateLayerRenderList();
        this.layerService.renderLayers();
      };
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
