var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/polygon/models/fill.ts
var fill_exports = {};
__export(fill_exports, {
  default: () => FillModel
});
module.exports = __toCommonJS(fill_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_polygon_frag = __toESM(require("../shaders/polygon_frag.glsl"));
var import_polygon_linear_frag = __toESM(require("../shaders/polygon_linear_frag.glsl"));
var import_polygon_linear_vert = __toESM(require("../shaders/polygon_linear_vert.glsl"));
var import_polygon_vert = __toESM(require("../shaders/polygon_vert.glsl"));
var FillModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      raisingHeight = 0,
      opacity = 1,
      opacityLinear = {
        enable: false,
        dir: "in"
      }
    } = this.layer.getLayerConfig();
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_raisingHeight: Number(raisingHeight),
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1,
      u_opacitylinear: Number(opacityLinear.enable),
      u_dir: opacityLinear.dir === "in" ? 1 : 0
    };
  }
  initModels(callbackModel) {
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    const { frag, vert, triangulation, type } = this.getModelParams();
    const {
      mask = false,
      maskInside = true,
      workerEnabled = false,
      enablePicking
    } = this.layer.getLayerConfig();
    this.layer.triangulation = triangulation;
    this.layer.buildLayerModel({
      moduleName: type,
      vertexShader: vert,
      fragmentShader: frag,
      triangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: false },
      blend: this.getBlend(),
      stencil: (0, import_l7_utils.getMask)(mask, maskInside),
      workerEnabled,
      workerOptions: {
        modelType: type,
        enablePicking
      }
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  clearModels() {
    var _a;
    (_a = this.dataTexture) == null ? void 0 : _a.destroy();
  }
  registerBuiltinAttributes() {
    const {
      opacityLinear = {
        enable: false,
        dir: "in"
      }
    } = this.layer.getLayerConfig();
    if (opacityLinear.enable) {
      this.styleAttributeService.registerStyleAttribute({
        name: "linear",
        type: import_l7_core.AttributeType.Attribute,
        descriptor: {
          name: "a_linear",
          buffer: {
            usage: import_l7_core.gl.STATIC_DRAW,
            data: [],
            type: import_l7_core.gl.FLOAT
          },
          size: 3,
          update: (feature, featureIdx, vertex, attributeIdx, normal) => {
            return [vertex[3], vertex[4], vertex[5]];
          }
        }
      });
    }
  }
  getModelParams() {
    const {
      opacityLinear = {
        enable: false
      }
    } = this.layer.getLayerConfig();
    if (opacityLinear.enable) {
      return {
        frag: import_polygon_linear_frag.default,
        vert: import_polygon_linear_vert.default,
        type: "polygonLinear",
        triangulation: import_triangulation.polygonTriangulationWithCenter
      };
    } else {
      return {
        frag: import_polygon_frag.default,
        vert: import_polygon_vert.default,
        type: "polygonFill",
        triangulation: import_l7_utils.polygonFillTriangulation
      };
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
