var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/polygon/models/water.ts
var water_exports = {};
__export(water_exports, {
  default: () => WaterModel
});
module.exports = __toCommonJS(water_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_polygon_water_frag = __toESM(require("../shaders/water/polygon_water_frag.glsl"));
var import_polygon_water_vert = __toESM(require("../shaders/water/polygon_water_vert.glsl"));
var WaterModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      speed = 0.5
    } = this.layer.getLayerConfig();
    if (this.dataTextureTest && this.dataTextureNeedUpdate({ opacity })) {
      this.judgeStyleAttributes({ opacity });
      const encodeData = this.layer.getEncodedData();
      const { data, width, height } = this.calDataFrame(this.cellLength, encodeData, this.cellProperties);
      this.rowCount = height;
      this.dataTexture = this.cellLength > 0 && data.length > 0 ? this.createTexture2D({
        flipY: true,
        data,
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width,
        height
      }) : this.createTexture2D({
        flipY: true,
        data: [1],
        format: import_l7_core.gl.LUMINANCE,
        type: import_l7_core.gl.FLOAT,
        width: 1,
        height: 1
      });
    }
    return {
      u_texture: this.texture,
      u_dataTexture: this.dataTexture,
      u_cellTypeLayout: this.getCellTypeLayout(),
      u_speed: speed,
      u_opacity: (0, import_lodash.isNumber)(opacity) ? opacity : 1
    };
  }
  getAnimateUniforms() {
    return {
      u_time: this.layer.getLayerAnimateTime()
    };
  }
  initModels(callbackModel) {
    this.loadTexture();
    this.buildModels(callbackModel);
  }
  buildModels(callbackModel) {
    const {
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    this.layer.buildLayerModel({
      moduleName: "polygonWater",
      vertexShader: import_polygon_water_vert.default,
      fragmentShader: import_polygon_water_frag.default,
      triangulation: import_triangulation.polygonTriangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: false },
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  clearModels() {
    var _a, _b;
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.dataTexture) == null ? void 0 : _b.destroy();
  }
  registerBuiltinAttributes() {
    const bbox = this.layer.getSource().extent;
    const [minLng, minLat, maxLng, maxLat] = bbox;
    const lngLen = maxLng - minLng;
    const latLen = maxLat - minLat;
    this.styleAttributeService.registerStyleAttribute({
      name: "waterUv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_uv",
        buffer: {
          usage: import_l7_core.gl.STATIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          const [lng, lat] = vertex;
          return [(lng - minLng) / lngLen, (lat - minLat) / latLen];
        }
      }
    });
  }
  loadTexture() {
    const {
      waterTexture
    } = this.layer.getLayerConfig();
    const { createTexture2D } = this.rendererService;
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    const image = new Image();
    image.crossOrigin = "";
    if (waterTexture) {
      console.warn("L7 recommend\uFF1Ahttps://gw.alipayobjects.com/mdn/rms_816329/afts/img/A*EojwT4VzSiYAAAAAAAAAAAAAARQnAQ");
      image.src = waterTexture;
    } else {
      image.src = "https://gw.alipayobjects.com/mdn/rms_816329/afts/img/A*EojwT4VzSiYAAAAAAAAAAAAAARQnAQ";
    }
    image.onload = () => {
      this.texture = createTexture2D({
        data: image,
        width: image.width,
        height: image.height,
        wrapS: import_l7_core.gl.MIRRORED_REPEAT,
        wrapT: import_l7_core.gl.MIRRORED_REPEAT,
        min: import_l7_core.gl.LINEAR,
        mag: import_l7_core.gl.LINEAR
      });
      this.layerService.updateLayerRenderList();
      this.layerService.renderLayers();
    };
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
