var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/raster/models/raster.ts
var raster_exports = {};
__export(raster_exports, {
  default: () => RasterModel
});
module.exports = __toCommonJS(raster_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_raster_2d_frag = __toESM(require("../shaders/raster_2d_frag.glsl"));
var import_raster_2d_vert = __toESM(require("../shaders/raster_2d_vert.glsl"));
var RasterModel = class extends import_BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      clampLow = true,
      clampHigh = true,
      noDataValue = -9999999,
      domain = [0, 1],
      rampColors
    } = this.layer.getLayerConfig();
    if (!(0, import_lodash.isEqual)(this.rampColors, rampColors)) {
      this.updateColorTexure();
      this.rampColors = rampColors;
    }
    return {
      u_opacity: opacity || 1,
      u_texture: this.texture,
      u_domain: domain,
      u_clampLow: clampLow,
      u_clampHigh: typeof clampHigh !== "undefined" ? clampHigh : clampLow,
      u_noDataValue: noDataValue,
      u_colorTexture: this.colorTexture
    };
  }
  initModels(callbackModel) {
    const {
      mask = false,
      maskInside = true,
      rampColorsData,
      rampColors
    } = this.layer.getLayerConfig();
    const source = this.layer.getSource();
    const { createTexture2D } = this.rendererService;
    const parserDataItem = source.data.dataArray[0];
    this.texture = createTexture2D({
      data: parserDataItem.data,
      width: parserDataItem.width,
      height: parserDataItem.height,
      format: import_l7_core.gl.LUMINANCE,
      type: import_l7_core.gl.FLOAT
    });
    const imageData = rampColorsData ? rampColorsData : (0, import_l7_utils.generateColorRamp)(rampColors);
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      flipY: false
    });
    this.layer.buildLayerModel({
      moduleName: "rasterImageData",
      vertexShader: import_raster_2d_vert.default,
      fragmentShader: import_raster_2d_frag.default,
      triangulation: import_triangulation.RasterImageTriangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: false },
      stencil: (0, import_l7_utils.getMask)(mask, maskInside)
    }).then((model) => {
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  buildModels(callbackModel) {
    this.initModels(callbackModel);
  }
  clearModels() {
    var _a, _b;
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.colorTexture) == null ? void 0 : _b.destroy();
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
  updateColorTexure() {
    const { createTexture2D } = this.rendererService;
    const {
      rampColors
    } = this.layer.getLayerConfig();
    const imageData = (0, import_l7_utils.generateColorRamp)(rampColors);
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      flipY: false
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
