var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/raster/raster.ts
var raster_exports = {};
__export(raster_exports, {
  default: () => RasterLayer
});
module.exports = __toCommonJS(raster_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_BaseLayer = __toESM(require("../core/BaseLayer"));
var import_triangulation = require("./buffers/triangulation");
var import_raster_frag = __toESM(require("./shaders/raster_frag.glsl"));
var import_raster_vert = __toESM(require("./shaders/raster_vert.glsl"));
var RasterLayer = class extends import_BaseLayer.default {
  constructor() {
    super(...arguments);
    this.type = "RasterLayer";
  }
  getAnimateUniforms() {
    return {};
  }
  buildModels() {
    const parserDataItem = this.getSource().data.dataArray[0];
    const { createTexture2D } = this.rendererService;
    this.rasterTexture = createTexture2D({
      data: parserDataItem.data,
      width: parserDataItem.width,
      height: parserDataItem.height,
      format: import_l7_core.gl.LUMINANCE,
      type: import_l7_core.gl.FLOAT,
      aniso: 4
    });
    const { rampColors } = this.getLayerConfig();
    const imageData = (0, import_l7_utils.generateColorRamp)(rampColors);
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      flipY: false
    });
    this.models = [this.buildRasterModel()];
  }
  renderModels() {
    const { opacity, heightRatio = 10 } = this.getLayerConfig();
    const parserDataItem = this.getSource().data.dataArray[0];
    const { coordinates, width, height, min, max } = parserDataItem;
    this.models.forEach((model) => model.draw({
      uniforms: {
        u_opacity: opacity || 1,
        u_texture: this.rasterTexture,
        u_min: min,
        u_width: width,
        u_height: height,
        u_max: max,
        u_heightRatio: heightRatio,
        u_colorTexture: this.colorTexture,
        u_extent: [...coordinates[0], ...coordinates[1]]
      }
    }));
    return this;
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  buildRasterModel() {
    const source = this.getSource();
    const sourceFeature = source.data.dataArray[0];
    const triangulation = (0, import_triangulation.RasterTriangulation)(sourceFeature);
    this.shaderModuleService.registerModule("raster", {
      vs: import_raster_vert.default,
      fs: import_raster_frag.default
    });
    const { vs, fs, uniforms } = this.shaderModuleService.getModule("raster");
    const {
      createAttribute,
      createElements,
      createBuffer,
      createModel
    } = this.rendererService;
    return createModel({
      vs,
      fs,
      attributes: {
        a_Position: createAttribute({
          buffer: createBuffer({
            data: triangulation.vertices,
            type: import_l7_core.gl.FLOAT
          }),
          size: 2
        })
      },
      primitive: import_l7_core.gl.TRIANGLES,
      uniforms: {
        ...uniforms
      },
      depth: {
        enable: true
      },
      elements: createElements({
        data: triangulation.indices,
        type: import_l7_core.gl.UNSIGNED_INT,
        count: triangulation.indices.length
      })
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
