var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/tile/tileFactory/base.ts
var base_exports = {};
__export(base_exports, {
  default: () => TileFactory
});
module.exports = __toCommonJS(base_exports);
var import_l7_source = __toESM(require("@antv/l7-source"));
var import_l7_utils = require("@antv/l7-utils");
var import_mask = __toESM(require("../../mask"));
var import_utils = require("../utils");
var import_vectorLayer = __toESM(require("./vectorLayer"));
var turf = __toESM(require("@turf/helpers"));
var import_union = __toESM(require("@turf/union"));
var TileFactory = class {
  constructor(option) {
    this.outSideEventTimer = null;
    this.eventCache = {
      click: 0,
      mousemove: 0,
      mouseup: 0,
      mousedown: 0,
      contextmenu: 0
    };
    this.parentLayer = option.parent;
    this.mapService = option.mapService;
    this.rendererService = option.rendererService;
    const source = this.parentLayer.getSource();
    this.zoomOffset = source.parser.zoomOffset || 0;
    this.tilesetManager = source.tileset;
  }
  createTile(tile, initOptions) {
    return {
      layers: [],
      layerIDList: []
    };
  }
  getFeatureData(tile, initOptions) {
    const emptyData = {
      features: [],
      featureId: null,
      vectorTileLayer: null,
      source: null
    };
    const { sourceLayer, featureId } = initOptions;
    if (!sourceLayer) {
      return emptyData;
    }
    const vectorTileLayer = tile.data.layers[sourceLayer];
    const features = vectorTileLayer == null ? void 0 : vectorTileLayer.features;
    if (!(Array.isArray(features) && features.length > 0)) {
      return emptyData;
    } else {
      const source = new import_l7_source.default({
        type: "FeatureCollection",
        features
      }, {
        parser: {
          type: "geojson",
          featureId
        }
      });
      return {
        features,
        featureId,
        vectorTileLayer,
        source
      };
    }
  }
  createLayer(tileLayerOption) {
    const {
      L7Layer,
      tile,
      initOptions,
      vectorTileLayer,
      source
    } = tileLayerOption;
    const { mask, color, layerType, size, shape } = initOptions;
    const FactoryTileLayer = L7Layer ? L7Layer : import_vectorLayer.default;
    const layer = new FactoryTileLayer({
      visible: tile.isVisible,
      tileOrigin: vectorTileLayer == null ? void 0 : vectorTileLayer.l7TileOrigin,
      coord: vectorTileLayer == null ? void 0 : vectorTileLayer.l7TileCoord,
      ...this.getLayerInitOption(initOptions)
    });
    if (layer.isVector) {
      this.emitEvent([layer]);
      layer.type = layerType;
      layer.select(true);
    }
    layer.source(source);
    this.setScale(layer);
    this.setStyleAttributeField(layer, "shape", shape);
    this.setStyleAttributeField(layer, "color", color);
    this.setStyleAttributeField(layer, "size", size);
    const layers = [layer];
    if (mask && layer.isVector) {
      const masklayer = new import_mask.default().source({
        type: "FeatureCollection",
        features: [tile.bboxPolygon]
      }).shape("fill");
      layers.push(masklayer);
      layer.addMaskLayer(masklayer);
    }
    (0, import_utils.registerLayers)(this.parentLayer, layers);
    this.layers = [layer];
    return layer;
  }
  updateStyle(styles) {
    return "";
  }
  getDefautStyleAttributeField(layer, type) {
    switch (type) {
      case "size":
        return 2;
      case "color":
        return "#fff";
      case "shape":
        return (0, import_utils.getLayerShape)(this.parentLayer.type, layer);
      default:
        return "";
    }
  }
  setStyleAttributeField(layer, type, value) {
    if (Array.isArray(value)) {
      layer[type](...value);
      return;
    }
    if (typeof value === "string") {
      layer[type](value);
      return;
    }
    const defaultValue = this.getDefautStyleAttributeField(layer, type);
    if (!value) {
      layer[type](defaultValue);
      return layer;
    }
    const params = this.parseScaleValue(value, type);
    if (params.length === 0) {
      layer[type](defaultValue);
    } else {
      layer[type](...params);
    }
  }
  parseScaleValue(value, type) {
    if (type === "shape") {
      if (typeof value === "string") {
        return [value];
      } else if (value == null ? void 0 : value.field) {
        return [value == null ? void 0 : value.field];
      } else {
        return [];
      }
    }
    const { field, values, callback } = value;
    if (field && values && Array.isArray(values)) {
      return [field, values];
    } else if (field && callback) {
      return [field, callback];
    } else if (field) {
      return [field];
    }
    return [];
  }
  getTile(lng, lat) {
    const zoom = this.mapService.getZoom();
    const z = Math.ceil(zoom) + this.zoomOffset;
    const xy = (0, import_l7_utils.osmLonLat2TileXY)(lng, lat, z);
    const tiles = this.tilesetManager.tiles.filter((t) => t.key === `${xy[0]},${xy[1]},${z}`);
    const tile = tiles[0];
    return tile;
  }
  emitEvent(layers, isVector) {
    layers.map((layer) => {
      layer.once("inited", () => {
        layer.on("click", (e) => {
          this.eventCache.click = 1;
          if (this.parentLayer.type === "RasterLayer") {
            const { lng, lat } = e.lngLat;
            const tile = this.getTile(lng, lat);
            this.getFeatureAndEmitEvent(layer, "subLayerClick", e, isVector, tile);
          } else {
            this.getFeatureAndEmitEvent(layer, "subLayerClick", e);
          }
        });
        layer.on("mousemove", (e) => {
          this.eventCache.mousemove = 1;
          if (this.parentLayer.type === "RasterLayer") {
            const { lng, lat } = e.lngLat;
            const tile = this.getTile(lng, lat);
            this.getFeatureAndEmitEvent(layer, "subLayerMouseMove", e, isVector, tile);
          } else {
            this.getFeatureAndEmitEvent(layer, "subLayerMouseMove", e);
          }
        });
        layer.on("mouseup", (e) => {
          this.eventCache.mouseup = 1;
          this.getFeatureAndEmitEvent(layer, "subLayerMouseUp", e);
        });
        layer.on("mouseenter", (e) => {
          if (this.parentLayer.type === "RasterLayer") {
            const { lng, lat } = e.lngLat;
            const tile = this.getTile(lng, lat);
            this.getFeatureAndEmitEvent(layer, "subLayerMouseMove", e, isVector, tile);
          } else {
            this.getFeatureAndEmitEvent(layer, "subLayerMouseEnter", e);
          }
        });
        layer.on("mouseout", (e) => {
          this.getFeatureAndEmitEvent(layer, "subLayerMouseOut", e);
        });
        layer.on("mousedown", (e) => {
          this.eventCache.mousedown = 1;
          this.getFeatureAndEmitEvent(layer, "subLayerMouseDown", e);
        });
        layer.on("contextmenu", (e) => {
          this.eventCache.contextmenu = 1;
          this.getFeatureAndEmitEvent(layer, "subLayerContextmenu", e);
        });
        layer.on("unclick", (e) => this.handleOutsideEvent("click", "subLayerUnClick", layer, e));
        layer.on("unmouseup", (e) => this.handleOutsideEvent("mouseup", "subLayerUnMouseUp", layer, e));
        layer.on("unmousedown", (e) => this.handleOutsideEvent("mousedown", "subLayerUnMouseDown", layer, e));
        layer.on("uncontextmenu", (e) => this.handleOutsideEvent("contextmenu", "subLayerUnContextmenu", layer, e));
      });
    });
  }
  getCombineFeature(features) {
    let p = null;
    const properties = features[0];
    features.map((feature) => {
      const polygon2 = turf.polygon(feature.coordinates);
      if (p === null) {
        p = polygon2;
      }
      {
        p = (0, import_union.default)(p, polygon2);
      }
    });
    if (properties) {
      p.properties = { ...properties };
    }
    return p;
  }
  getFeatureAndEmitEvent(layer, eventName, e, isVector, tile) {
    if (isVector === false) {
      e.value = (0, import_utils.readRasterValue)(tile, this.mapService, e.x, e.y);
    } else {
      const featureId = e.featureId;
      const features = this.getAllFeatures(featureId);
      try {
        e.feature = this.getCombineFeature(features);
      } catch (err) {
        console.warn("Combine Featuer Err! Return First Feature!");
        e.feature = features[0];
      }
    }
    this.parentLayer.emit(eventName, e);
  }
  setScale(layer) {
    const scaleOptions = this.parentLayer.tileLayer.scaleField;
    const scaleKeys = Object.keys(scaleOptions);
    scaleKeys.map((key) => {
      layer.scale(key, scaleOptions[key]);
    });
  }
  getAllFeatures(featureId) {
    const allLayers = this.parentLayer.tileLayer.children;
    const features = [];
    allLayers.map((layer) => {
      const source = layer.getSource();
      source.data.dataArray.map((feature) => {
        if (feature._id === featureId) {
          features.push(feature);
        }
      });
    });
    return features;
  }
  getLayerInitOption(initOptions) {
    const option = { ...initOptions };
    delete option.color;
    delete option.shape;
    delete option.size;
    delete option.coords;
    delete option.sourceLayer;
    delete option.coords;
    return option;
  }
  handleOutsideEvent(type, emitType, layer, e) {
    if (this.outSideEventTimer) {
      clearTimeout(this.outSideEventTimer);
      this.outSideEventTimer = null;
    }
    this.outSideEventTimer = setTimeout(() => {
      if (this.eventCache[type] > 0) {
        this.eventCache[type] = 0;
      } else {
        this.getFeatureAndEmitEvent(layer, emitType, e);
      }
    }, 64);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
