var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/tile/tileLayer/baseTileLayer.ts
var baseTileLayer_exports = {};
__export(baseTileLayer_exports, {
  default: () => BaseTileLayer
});
module.exports = __toCommonJS(baseTileLayer_exports);
var import_l7_utils = require("@antv/l7-utils");
var import_tileLayerManager = require("../manager/tileLayerManager");
var BaseTileLayer = class {
  constructor({
    parent,
    rendererService,
    mapService,
    layerService,
    pickingService
  }) {
    this.type = "baseTile";
    this.initedTileset = false;
    this.pickColors = {
      select: null,
      active: null
    };
    var _a;
    const parentSource = parent.getSource();
    const { sourceLayer, coords, featureId } = ((_a = parentSource == null ? void 0 : parentSource.data) == null ? void 0 : _a.tilesetOptions) || {};
    this.sourceLayer = sourceLayer;
    this.parent = parent;
    this.mapService = mapService;
    this.layerService = layerService;
    this.tileLayerManager = new import_tileLayerManager.TileLayerManager(parent, mapService, rendererService, pickingService, layerService);
    this.initTileSetManager();
    this.bindSubLayerEvent();
    this.bindSubLayerPick();
    this.scaleField = this.parent.getScaleOptions();
  }
  get children() {
    return this.tileLayerManager.children;
  }
  render() {
    if (this.tileLayerManager) {
      this.tileLayerManager.render();
    }
  }
  clearPick(type) {
    if (type === "mousemove") {
      this.tileLayerManager.tilePickManager.clearPick();
    }
  }
  clearPickState() {
    this.children.filter((child) => child.inited && child.isVisible()).filter((child) => child.getCurrentSelectedId() !== null).map((child) => {
      this.selectFeature(child, new Uint8Array([0, 0, 0, 0]));
      child.setCurrentSelectedId(null);
    });
  }
  pickLayers(target) {
    return this.tileLayerManager.pickLayers(target);
  }
  tileLoaded(tile) {
  }
  tileError(error) {
    console.warn("error:", error);
  }
  tileUnLoad(tile) {
    this.tileLayerManager.removeChilds(tile.layerIDList, false);
  }
  tileUpdate() {
    if (!this.tilesetManager) {
      return;
    }
    this.tilesetManager.tiles.filter((tile) => tile.isLoaded).map((tile) => {
      if (tile.layerIDList.length === 0) {
        const { layers, layerIDList } = this.tileLayerManager.createTile(tile);
        tile.layerIDList = layerIDList;
        this.tileLayerManager.addChilds(layers);
      } else {
        if (!tile.isVisibleChange) {
          return;
        }
        const layers = this.tileLayerManager.getChilds(tile.layerIDList);
        this.tileLayerManager.updateLayersConfig(layers, "visible", tile.isVisible);
      }
    });
    if (this.tilesetManager.isLoaded) {
      this.parent.emit("tiles-loaded", this.tilesetManager.currentTiles);
    }
  }
  setPickState(layers) {
    if (this.pickColors.select) {
      const selectedId = (0, import_l7_utils.decodePickingColor)(this.pickColors.select);
      layers.map((layer) => {
        this.selectFeature(layer, this.pickColors.select);
        layer.setCurrentSelectedId(selectedId);
      });
    }
    if (this.pickColors.active) {
      const selectedId = (0, import_l7_utils.decodePickingColor)(this.pickColors.active);
      layers.filter((layer) => layer.inited && layer.isVisible()).map((layer) => {
        layer.hooks.beforeHighlight.call(this.pickColors.active);
        layer.setCurrentPickId(selectedId);
      });
    }
  }
  bindSubLayerPick() {
    this.tileLayerManager.tilePickManager.on("pick", (e) => {
      const [r, g, b] = e.pickedColors;
      if (e.type === "click") {
        const restLayers = this.children.filter((child) => child.inited && child.isVisible() && child.isVector).filter((child) => child !== e.layer);
        this.setSelect(restLayers, [r, g, b]);
      } else {
        this.setHighlight([r, g, b]);
      }
    });
    this.tileLayerManager.tilePickManager.on("unpick", () => {
      this.pickColors.active = null;
    });
  }
  setHighlight(pickedColors) {
    const pickId = (0, import_l7_utils.decodePickingColor)(pickedColors);
    this.pickColors.active = pickedColors;
    this.children.filter((child) => child.inited && child.isVisible() && child.isVector).filter((child) => child.getPickID() !== pickId).map((child) => {
      child.setPickID(pickId);
      child.hooks.beforeHighlight.call(pickedColors);
    });
  }
  setSelect(layers, pickedColors) {
    const selectedId = (0, import_l7_utils.decodePickingColor)(pickedColors);
    layers.map((layer) => {
      if (layer.getCurrentSelectedId() === null || selectedId !== layer.getCurrentSelectedId()) {
        this.selectFeature(layer, pickedColors);
        layer.setCurrentSelectedId(selectedId);
        this.pickColors.select = pickedColors;
      } else {
        this.selectFeature(layer, new Uint8Array([0, 0, 0, 0]));
        layer.setCurrentSelectedId(null);
        this.pickColors.select = null;
      }
    });
    const renderList = this.layerService.getRenderList();
    renderList.filter((layer) => layer.inited && !layer.isVector && layer.isVisible() && layer.needPick("click")).filter((layer) => layer.getCurrentSelectedId() !== null).map((layer) => {
      this.selectFeature(layer, new Uint8Array([0, 0, 0, 0]));
      layer.setCurrentSelectedId(null);
    });
  }
  selectFeature(layer, pickedColors) {
    const [r, g, b] = pickedColors;
    layer.hooks.beforeSelect.call([r, g, b]);
  }
  bindSubLayerEvent() {
    this.parent.on("subLayerClick", (e) => {
      this.parent.emit("click", { ...e });
    });
    this.parent.on("subLayerMouseMove", (e) => this.parent.emit("mousemove", { ...e }));
    this.parent.on("subLayerMouseUp", (e) => this.parent.emit("mouseup", { ...e }));
    this.parent.on("subLayerMouseEnter", (e) => this.parent.emit("mouseenter", { ...e }));
    this.parent.on("subLayerMouseOut", (e) => this.parent.emit("mouseout", { ...e }));
    this.parent.on("subLayerMouseDown", (e) => this.parent.emit("mousedown", { ...e }));
    this.parent.on("subLayerContextmenu", (e) => this.parent.emit("contextmenu", { ...e }));
    this.parent.on("subLayerUnClick", (e) => this.parent.emit("unclick", { ...e }));
    this.parent.on("subLayerUnMouseMove", (e) => this.parent.emit("unmousemove", { ...e }));
    this.parent.on("subLayerUnMouseUp", (e) => this.parent.emit("unmouseup", { ...e }));
    this.parent.on("subLayerUnMouseDown", (e) => this.parent.emit("unmousedown", { ...e }));
    this.parent.on("subLayerUnContextmenu", (e) => this.parent.emit("uncontextmenu", { ...e }));
  }
  initTileSetManager() {
    var _a;
    const source = this.parent.getSource();
    this.tilesetManager = source.tileset;
    if (!this.initedTileset) {
      this.bindTilesetEvent();
      this.initedTileset = true;
    }
    const { latLonBounds, zoom } = this.getCurrentView();
    (_a = this.tilesetManager) == null ? void 0 : _a.update(zoom, latLonBounds);
  }
  mapchange() {
    var _a;
    const { latLonBounds, zoom } = this.getCurrentView();
    if (this.mapService.version === "GAODE1.x") {
      const { visible } = this.parent.getLayerConfig();
      if (zoom < 3 && visible) {
        this.parent.updateLayerConfig({ visible: false });
        this.layerService.updateLayerRenderList();
      } else if (zoom >= 3 && !visible) {
        this.parent.updateLayerConfig({ visible: true });
        this.layerService.updateLayerRenderList();
      }
    }
    if (this.lastViewStates && this.lastViewStates.zoom === zoom && this.lastViewStates.latLonBounds.toString() === latLonBounds.toString()) {
      return;
    }
    this.lastViewStates = { zoom, latLonBounds };
    if (this.timer) {
      clearTimeout(this.timer);
      this.timer = null;
    }
    (_a = this.tilesetManager) == null ? void 0 : _a.update(zoom, latLonBounds);
  }
  bindTilesetEvent() {
    if (!this.tilesetManager) {
      return;
    }
    this.tilesetManager.on("tile-loaded", (tile) => {
    });
    this.tilesetManager.on("tile-unload", (tile) => {
      this.tileUnLoad(tile);
    });
    this.tilesetManager.on("tile-error", (error, tile) => {
      this.tileError(error);
    });
    this.tilesetManager.on("tile-update", () => {
      this.tileUpdate();
    });
    this.mapService.on("zoomend", () => this.mapchange());
    this.mapService.on("moveend", () => this.mapchange());
  }
  getCurrentView() {
    const bounds = this.mapService.getBounds();
    const latLonBounds = [
      bounds[0][0],
      bounds[0][1],
      bounds[1][0],
      bounds[1][1]
    ];
    const zoom = this.mapService.getZoom();
    return { latLonBounds, zoom };
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
