var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/tile/utils.ts
var utils_exports = {};
__export(utils_exports, {
  getContainerSize: () => getContainerSize,
  getLayerShape: () => getLayerShape,
  getMaskValue: () => getMaskValue,
  readPixel: () => readPixel,
  readRasterValue: () => readRasterValue,
  registerLayers: () => registerLayers
});
module.exports = __toCommonJS(utils_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
function registerLayers(parentLayer, layers) {
  layers.map((layer) => {
    const container = (0, import_l7_core.createLayerContainer)(parentLayer.sceneContainer);
    layer.setContainer(container, parentLayer.sceneContainer);
    layer.init();
  });
}
function getLayerShape(layerType, layer) {
  var _a, _b;
  const layerShape = layer.getAttribute("shape");
  if (layerShape && ((_a = layerShape.scale) == null ? void 0 : _a.field)) {
    if (((_b = layerShape.scale) == null ? void 0 : _b.values) === "text") {
      return [layerShape.scale.field, layerShape.scale.values];
    }
    return layerShape.scale.field;
  }
  switch (layerType) {
    case "PolygonLayer":
      return "fill";
    case "LineLayer":
      return "tileline";
    case "PointLayer":
      return "circle";
    case "RasterLayer":
      return "image";
    default:
      return "";
  }
}
function getMaskValue(layerType, mask) {
  switch (layerType) {
    case "PolygonLayer":
      return true;
    case "LineLayer":
      return true;
    case "PointLayer":
      return false;
    case "RasterLayer":
      return mask;
    default:
      return mask;
  }
}
function getContainerSize(container) {
  if (!!container.getContext) {
    return {
      width: container.width / import_l7_utils.DOM.DPR,
      height: container.height / import_l7_utils.DOM.DPR
    };
  } else {
    return container.getBoundingClientRect();
  }
}
function readRasterValue(tile, mapService, x, y) {
  var _a, _b, _c, _d;
  const bbox = ((_a = tile == null ? void 0 : tile.bboxPolygon) == null ? void 0 : _a.bbox) || [0, 0, 10, -10];
  const [minLng = 0, minLat = 0, maxLng = 10, maxLat = -10] = bbox;
  const tileXY = mapService.lngLatToContainer([minLng, minLat]);
  const tileMaxXY = mapService.lngLatToContainer([maxLng, maxLat]);
  const tilePixelWidth = tileMaxXY.x - tileXY.x;
  const tilePixelHeight = tileXY.y - tileMaxXY.y;
  const pos = [
    (x - tileXY.x) / tilePixelWidth,
    (y - tileMaxXY.y) / tilePixelHeight
  ];
  const tileWidth = ((_b = tile == null ? void 0 : tile.data) == null ? void 0 : _b.width) || 1;
  const tileHeight = ((_c = tile == null ? void 0 : tile.data) == null ? void 0 : _c.height) || 1;
  const indexX = Math.floor(pos[0] * tileWidth);
  const indexY = Math.floor(pos[1] * tileHeight);
  const index = Math.max(0, indexY - 1) * tileWidth + indexX;
  const data = (_d = tile == null ? void 0 : tile.data) == null ? void 0 : _d.data[index];
  return data;
}
function readPixel(x, y, rendererService) {
  const { readPixels, getContainer } = rendererService;
  const xInDevicePixel = x * import_l7_utils.DOM.DPR;
  const yInDevicePixel = y * import_l7_utils.DOM.DPR;
  let { width, height } = getContainerSize(getContainer());
  width *= import_l7_utils.DOM.DPR;
  height *= import_l7_utils.DOM.DPR;
  if (xInDevicePixel > width - 1 * import_l7_utils.DOM.DPR || xInDevicePixel < 0 || yInDevicePixel > height - 1 * import_l7_utils.DOM.DPR || yInDevicePixel < 0) {
    return false;
  }
  const pickedColors = readPixels({
    x: Math.floor(xInDevicePixel),
    y: Math.floor(height - (y + 1) * import_l7_utils.DOM.DPR),
    width: 1,
    height: 1,
    data: new Uint8Array(1 * 1 * 4)
  });
  return pickedColors;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getContainerSize,
  getLayerShape,
  getMaskValue,
  readPixel,
  readRasterValue,
  registerLayers
});
