var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/wind/models/wind.ts
var wind_exports = {};
__export(wind_exports, {
  default: () => WindModel
});
module.exports = __toCommonJS(wind_exports);
var import_l7_core = require("@antv/l7-core");
var import_l7_utils = require("@antv/l7-utils");
var import_BaseModel = __toESM(require("../../core/BaseModel"));
var import_triangulation = require("../../core/triangulation");
var import_wind_frag = __toESM(require("../shaders/wind_frag.glsl"));
var import_wind_vert = __toESM(require("../shaders/wind_vert.glsl"));
var import_windRender = require("./windRender");
var defaultRampColors = {
  0: "#3288bd",
  0.1: "#66c2a5",
  0.2: "#abdda4",
  0.3: "#e6f598",
  0.4: "#fee08b",
  0.5: "#fdae61",
  0.6: "#f46d43",
  1: "#d53e4f"
};
var WindModel = class extends import_BaseModel.default {
  constructor() {
    super(...arguments);
    this.sizeScale = 0.5;
    this.frequency = new import_l7_utils.FrequencyController(7.2);
  }
  render() {
    this.frequency.run(() => {
      this.drawWind();
    });
    this.drawColorMode();
  }
  getUninforms() {
    throw new Error("Method not implemented.");
  }
  initModels(callbackModel) {
    var _a, _b, _c;
    const {
      uMin = -21.32,
      uMax = 26.8,
      vMin = -21.57,
      vMax = 21.42,
      fadeOpacity = 0.996,
      speedFactor = 0.25,
      dropRate = 3e-3,
      dropRateBump = 0.01,
      rampColors = defaultRampColors,
      sizeScale = 0.5,
      mask = false,
      maskInside = true
    } = this.layer.getLayerConfig();
    const { createTexture2D } = this.rendererService;
    const source = this.layer.getSource();
    this.texture = createTexture2D({
      height: 0,
      width: 0
    });
    this.cacheZoom = Math.floor(this.mapService.getZoom());
    const glContext = this.rendererService.getGLContext();
    this.imageCoords = (_a = source.data) == null ? void 0 : _a.dataArray[0].coordinates;
    (_c = (_b = source.data) == null ? void 0 : _b.images) == null ? void 0 : _c.then((imageData) => {
      var _a2;
      this.sizeScale = sizeScale * this.getZoomScale();
      const { imageWidth, imageHeight } = this.getWindSize();
      const options = {
        glContext,
        imageWidth,
        imageHeight,
        fadeOpacity,
        speedFactor,
        dropRate,
        dropRateBump,
        rampColors
      };
      this.wind = new import_windRender.Wind(options);
      this.wind.setWind({
        uMin,
        uMax,
        vMin,
        vMax,
        image: imageData[0]
      });
      (_a2 = this.texture) == null ? void 0 : _a2.destroy();
      this.texture = createTexture2D({
        width: imageWidth,
        height: imageHeight
      });
      this.layerService.updateLayerRenderList();
      this.layerService.renderLayers();
    });
    this.layer.buildLayerModel({
      moduleName: "wind",
      vertexShader: import_wind_vert.default,
      fragmentShader: import_wind_frag.default,
      triangulation: import_triangulation.RasterImageTriangulation,
      primitive: import_l7_core.gl.TRIANGLES,
      depth: { enable: false },
      blend: this.getBlend()
    }).then((model) => {
      this.colorModel = model;
      callbackModel([model]);
    }).catch((err) => {
      console.warn(err);
      callbackModel([]);
    });
  }
  getWindSize() {
    const p1 = this.mapService.lngLatToPixel(this.imageCoords[0]);
    const p2 = this.mapService.lngLatToPixel(this.imageCoords[1]);
    const imageWidth = Math.min(Math.floor((p2.x - p1.x) * this.sizeScale), 2048);
    const imageHeight = Math.min(Math.floor((p1.y - p2.y) * this.sizeScale), 2048);
    return { imageWidth, imageHeight };
  }
  buildModels(callbackModel) {
    this.initModels(callbackModel);
  }
  clearModels() {
    var _a, _b;
    (_a = this.texture) == null ? void 0 : _a.destroy();
    (_b = this.wind) == null ? void 0 : _b.destroy();
  }
  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: "number",
          minimum: 0,
          maximum: 1
        }
      }
    };
  }
  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: "uv",
      type: import_l7_core.AttributeType.Attribute,
      descriptor: {
        name: "a_Uv",
        buffer: {
          usage: import_l7_core.gl.DYNAMIC_DRAW,
          data: [],
          type: import_l7_core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }
  getZoomScale() {
    return Math.min((this.cacheZoom + 4) / 30 * 2, 2);
  }
  drawWind() {
    if (this.wind) {
      const {
        uMin = -21.32,
        uMax = 26.8,
        vMin = -21.57,
        vMax = 21.42,
        numParticles = 65535,
        fadeOpacity = 0.996,
        speedFactor = 0.25,
        dropRate = 3e-3,
        dropRateBump = 0.01,
        rampColors = defaultRampColors,
        sizeScale = 0.5
      } = this.layer.getLayerConfig();
      let newNumParticles = numParticles;
      const currentZoom = Math.floor(this.mapService.getZoom());
      if (typeof sizeScale === "number" && sizeScale !== this.sizeScale || currentZoom !== this.cacheZoom) {
        const zoomScale = this.getZoomScale();
        this.sizeScale = sizeScale;
        newNumParticles *= zoomScale;
        const { imageWidth, imageHeight } = this.getWindSize();
        this.wind.reSize(imageWidth, imageHeight);
        this.cacheZoom = currentZoom;
      }
      this.wind.updateWindDir(uMin, uMax, vMin, vMax);
      this.wind.updateParticelNum(newNumParticles);
      this.wind.updateColorRampTexture(rampColors);
      this.wind.fadeOpacity = fadeOpacity;
      this.wind.speedFactor = speedFactor;
      this.wind.dropRate = dropRate;
      this.wind.dropRateBump = dropRateBump;
      const { d, w, h } = this.wind.draw();
      this.rendererService.setBaseState();
      this.texture.update({
        data: d,
        width: w,
        height: h
      });
    }
  }
  drawColorMode() {
    var _a;
    const { opacity } = this.layer.getLayerConfig();
    this.layer.masks.map((m) => {
      m.hooks.beforeRenderData.call();
      m.hooks.beforeRender.call();
      m.render();
      m.hooks.afterRender.call();
    });
    (_a = this.colorModel) == null ? void 0 : _a.draw({
      uniforms: {
        u_opacity: opacity || 1,
        u_texture: this.texture
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
